@@ -1,25 +1,26 @@
 from __future__ import annotations
 
 from collections.abc import Iterator
-from dataclasses import dataclass
-from dataclasses import field
+from dataclasses import dataclass, field
 from itertools import chain
 from pathlib import Path
-from typing import TYPE_CHECKING
+from typing import Self
 
 import numpy as np
-from lmfit.parameter import Parameters as ParametersLF
+from lmfit import Parameters as ParametersLF
 
+from chemex.configuration.methods import Selection
 from chemex.containers.profile import Profile
+from chemex.messages import (
+    print_no_duplicate_warning,
+    print_not_implemented_noise_method_warning,
+)
 from chemex.parameters import database
 from chemex.parameters.spin_system import Group
-from chemex.plotters import Plotter
+from chemex.plotters.plotter import Plotter
 from chemex.printers.data import Printer
 from chemex.uncertainty import estimate_noise_variance
 
-if TYPE_CHECKING:
-    from chemex.configuration.methods import Selection
-
 
 @dataclass
 class Experiment:
@@ -32,7 +33,7 @@ class Experiment:
 
     def residuals(self, params: ParametersLF) -> list[float]:
         return list(
-            chain.from_iterable(profile.residuals(params) for profile in self.profiles)
+            chain.from_iterable(profile.residuals(params) for profile in self.profiles),
         )
 
     def plot(self, path: Path):
@@ -43,7 +44,7 @@ class Experiment:
 
     def write(self, path: Path):
         filename = (path / self.filename.name).with_suffix(".dat")
-        with filename.open("w") as file_dat:
+        with filename.open("w", encoding="utf-8") as file_dat:
             file_dat.write(self.printer.header)
             for profile in sorted(self.profiles):
                 file_dat.write(str(profile))
@@ -52,10 +53,11 @@ class Experiment:
         include = selection.include
         exclude = selection.exclude
         profiles_all = [*self.profiles, *self.filtered_profiles]
-        profiles, filtered = [], []
+        profiles: list[Profile] = []
+        filtered: list[Profile] = []
         for profile in profiles_all:
-            included = include is None or profile.name.part_of(include)
-            excluded = exclude is not None and profile.name.part_of(exclude)
+            included = include is None or profile.spin_system.part_of(include)
+            excluded = exclude is not None and profile.spin_system.part_of(exclude)
             if included and not excluded:
                 profiles.append(profile)
             else:
@@ -70,63 +72,71 @@ class Experiment:
     def _any_duplicate(self):
         return any(profile.any_duplicate() for profile in self.profiles)
 
-    def estimate_noise(self, kind: str):
+    def estimate_noise(self, kind: str, global_error: bool = True) -> None:
         # TODO: Validation should be moved to the configuration file module
         implemented = ("file", "scatter", "duplicates")
         if kind not in implemented:
-            print(
-                f"Warning: Experiment {self.filename.name}: The method '{kind}' is not "
-                f"implemented. Please choose one of the following methods: "
-                f"{implemented}"
-            )
+            print_not_implemented_noise_method_warning(self.filename, kind, implemented)
             kind = "file"
         if kind == "duplicates" and not self._any_duplicate():
-            print(
-                f"Warning: Experiment {self.filename.name}: Some profiles have no "
-                f"duplicate points: Uncertainties are not estimated and directly taken "
-                f"from the files."
-            )
+            print_no_duplicate_warning(self.filename)
             kind = "file"
         if kind == "file" or not self.profiles:
             return
-        noise_variance_values = [
-            estimate_noise_variance[kind](profile.data) for profile in self.profiles
-        ]
-        noise_mean = np.sqrt(np.mean(noise_variance_values))
-        for profile in self.profiles:
-            profile.set_noise(noise_mean)
+        if global_error:
+            noise_variance_values = [
+                estimate_noise_variance[kind](profile.data) for profile in self.profiles
+            ]
+            noise_mean = np.sqrt(np.mean(noise_variance_values))
+            for profile in self.profiles:
+                profile.set_noise(noise_mean)
+        else:
+            for profile in self.profiles:
+                profile.set_noise(np.sqrt(estimate_noise_variance[kind](profile.data)))
 
     def prepare_for_simulation(self) -> None:
         for profile in self.profiles:
             profile.prepare_for_simulation()
 
-    def monte_carlo(self) -> Experiment:
+    def monte_carlo(self) -> Self:
         profiles = [profile.monte_carlo() for profile in self.profiles]
-        return Experiment(
-            self.filename, self.name, profiles, self.printer, self.plotter
+        return type(self)(
+            self.filename,
+            self.name,
+            profiles,
+            self.printer,
+            self.plotter,
         )
 
-    def bootstrap(self) -> Experiment:
+    def bootstrap(self) -> Self:
         profiles = [profile.bootstrap() for profile in self.profiles]
-        return Experiment(
-            self.filename, self.name, profiles, self.printer, self.plotter
+        return type(self)(
+            self.filename,
+            self.name,
+            profiles,
+            self.printer,
+            self.plotter,
         )
 
-    def bootstrap_ns(self, groups: list[Group]) -> Experiment:
+    def bootstrap_ns(self, groups: list[Group]) -> Self:
         """Residue-specific bootstrap."""
-        profiles = {}
+        profiles: dict[Group, list[Profile]] = {}
         for profile in self.profiles:
-            profiles.setdefault(profile.name.groups["i"], []).append(profile)
-        profiles_bs_ns = []
+            profiles.setdefault(profile.spin_system.groups["i"], []).append(profile)
+        profiles_bs_ns: list[Profile] = []
         for group in groups:
             profiles_bs_ns.extend(profiles.get(group, []))
-        return Experiment(
-            self.filename, self.name, profiles_bs_ns, self.printer, self.plotter
+        return type(self)(
+            self.filename,
+            self.name,
+            profiles_bs_ns,
+            self.printer,
+            self.plotter,
         )
 
     @property
     def groups(self) -> set[Group]:
-        return {profile.name.groups["i"] for profile in self.profiles}
+        return {profile.spin_system.groups["i"] for profile in self.profiles}
 
     @property
     def param_id_sets(self) -> list[set[str]]:
@@ -134,15 +144,19 @@ class Experiment:
             set(database.get_parameters(profile.param_ids)) for profile in self.profiles
         ]
 
-    def get_relevant_subset(self, param_ids: set[str]) -> Experiment:
+    def get_relevant_subset(self, param_ids: set[str]) -> Self:
         profiles = [
             profile
             for profile in self.profiles
             if set(database.get_parameters(profile.param_ids)) & param_ids
         ]
 
-        return Experiment(
-            self.filename, self.name, profiles, self.printer, self.plotter
+        return type(self)(
+            self.filename,
+            self.name,
+            profiles,
+            self.printer,
+            self.plotter,
         )
 
     def __iter__(self) -> Iterator[Profile]:
