@@ -4,31 +4,26 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettings
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.constants import get_multiplet
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cest_13c"
+OFFSET_REF = 1e4
 
 
 class Cest13CSettings(CestSettings):
@@ -45,17 +40,21 @@ class Cest13CSettings(CestSettings):
         return f"[iz_{self.observed_state}]"
 
 
-class Cest13CConfig(ExperimentConfig[Cest13CSettings, CestDataSettings]):
+class Cest13CConfig(
+    ExperimentConfiguration[
+        Cest13CSettings, ConditionsWithValidations, CestDataSettings
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
         return ToBeFitted(
-            rates=["r2_i", f"r1_i_{state}"], model_free=[f"tauc_{state}", f"s2_{state}"]
+            rates=["r2_i", f"r1_i_{state}"],
+            model_free=[f"tauc_{state}", f"s2_{state}"],
         )
 
 
 def build_spectrometer(config: Cest13CConfig, spin_system: SpinSystem) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -83,19 +82,17 @@ class Cest13CSequence:
     settings: Cest13CSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
-
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
 
         start = spectrometer.get_equilibrium()
 
-        intensities = {}
+        intensities: dict[float, ArrayFloat] = {}
 
         for offset in set(offsets):
-
             intensities[offset] = start
 
             if self.is_reference(offset):
@@ -108,7 +105,7 @@ class Cest13CSequence:
             )
 
         return np.array(
-            [spectrometer.detect(intensities[offset]) for offset in offsets]
+            [spectrometer.detect(intensities[offset]) for offset in offsets],
         )
 
 
