@@ -4,32 +4,28 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettings
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.constants import get_multiplet
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cest_15n_cw"
 
+OFFSET_REF = 1e4
+
 
 class Cest15NCwSettings(CestSettings):
     name: Literal["cest_15n_cw"]
@@ -40,14 +36,17 @@ class Cest15NCwSettings(CestSettings):
     b1_frq: float
     b1_inh_scale: float = 0.1
     b1_inh_res: int = 11
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
         return f"[iz_{self.observed_state}]"
 
 
-class Cest15NCwConfig(ExperimentConfig[Cest15NCwSettings, CestDataSettings]):
+class Cest15NCwConfig(
+    ExperimentConfiguration[
+        Cest15NCwSettings, ConditionsWithValidations, CestDataSettings
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -66,9 +65,9 @@ class Cest15NCwConfig(ExperimentConfig[Cest15NCwSettings, CestDataSettings]):
 
 
 def build_spectrometer(
-    config: Cest15NCwConfig, spin_system: SpinSystem
+    config: Cest15NCwConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -96,19 +95,17 @@ class Cest15NCwSequence:
     settings: Cest15NCwSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
-
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
 
         start = spectrometer.get_start_magnetization(terms=["iz"], atom="n")
 
-        intensities = {}
+        intensities: dict[float, ArrayFloat] = {}
 
         for offset in set(offsets):
-
             intensities[offset] = start
 
             if self.is_reference(offset):
@@ -122,7 +119,7 @@ class Cest15NCwSequence:
             )
 
         return np.array(
-            [spectrometer.detect(intensities[offset]) for offset in offsets]
+            [spectrometer.detect(intensities[offset]) for offset in offsets],
         )
 
 
