@@ -4,32 +4,28 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettings
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.constants import get_multiplet
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cest_15n_tr"
 
+OFFSET_REF = 1e4
+
 
 class Cest15NTrSettings(CestSettings):
     name: Literal["cest_15n_tr"]
@@ -39,17 +35,19 @@ class Cest15NTrSettings(CestSettings):
     b1_inh_scale: float = 0.1
     b1_inh_res: int = 11
     antitrosy: bool = False
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
         if self.antitrosy:
             return f"[2izsz_{self.observed_state}] + [iz_{self.observed_state}]"
-        else:
-            return f"[2izsz_{self.observed_state}] - [iz_{self.observed_state}]"
+        return f"[2izsz_{self.observed_state}] - [iz_{self.observed_state}]"
 
 
-class Cest15NTrConfig(ExperimentConfig[Cest15NTrSettings, CestDataSettings]):
+class Cest15NTrConfig(
+    ExperimentConfiguration[
+        Cest15NTrSettings, ConditionsWithValidations, CestDataSettings
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -66,9 +64,9 @@ class Cest15NTrConfig(ExperimentConfig[Cest15NTrSettings, CestDataSettings]):
 
 
 def build_spectrometer(
-    config: Cest15NTrConfig, spin_system: SpinSystem
+    config: Cest15NTrConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -94,29 +92,28 @@ class Cest15NTrSequence:
     settings: Cest15NTrSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
+
+    def _get_start(self, spectrometer: Spectrometer) -> ArrayFloat:
+        """Start from the TROSY or ANTI-TROSY component.
 
-    def _get_start(self, spectrometer: Spectrometer) -> np.ndarray:
-        """
         TROSY: (2IzSz - Iz) / 2
-        ANTITROSY: (2IzSz + Iz) / 2
+        ANTITROSY: (2IzSz + Iz) / 2.
         """
         start = 0.5 * spectrometer.get_start_magnetization(["2izsz", "iz"])
         if not self.settings.antitrosy:
             start -= spectrometer.get_start_magnetization(["iz"])
         return start
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
 
         start = self._get_start(spectrometer)
 
-        intensities = {}
+        intensities: dict[float, ArrayFloat] = {}
 
         for offset in set(offsets):
-
             intensities[offset] = start
 
             if self.is_reference(offset):
@@ -129,7 +126,7 @@ class Cest15NTrSequence:
             )
 
         return np.array(
-            [spectrometer.detect(intensities[offset]) for offset in offsets]
+            [spectrometer.detect(intensities[offset]) for offset in offsets],
         )
 
 
