@@ -4,31 +4,27 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettings
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cest_1hn_ap"
 
+OFFSET_REF = 1e4
+
 
 class Cest1HnApSettings(CestSettings):
     name: Literal["cest_1hn_ap"]
@@ -48,7 +44,11 @@ class Cest1HnApSettings(CestSettings):
         return f"[2izsz_{self.observed_state}]"
 
 
-class Cest1HnApConfig(ExperimentConfig[Cest1HnApSettings, CestDataSettings]):
+class Cest1HnApConfig(
+    ExperimentConfiguration[
+        Cest1HnApSettings, ConditionsWithValidations, CestDataSettings
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -59,9 +59,9 @@ class Cest1HnApConfig(ExperimentConfig[Cest1HnApSettings, CestDataSettings]):
 
 
 def build_spectrometer(
-    config: Cest1HnApConfig, spin_system: SpinSystem
+    config: Cest1HnApConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -84,18 +84,17 @@ class Cest1HnApSequence:
     settings: Cest1HnApSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
-
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
 
         start = spectrometer.get_start_magnetization(terms=self.settings.start)
 
-        intensities = {}
-        for offset in set(offsets):
+        intensities: dict[float, ArrayFloat] = {}
 
+        for offset in set(offsets):
             intensities[offset] = start
 
             if self.is_reference(offset):
@@ -108,7 +107,7 @@ class Cest1HnApSequence:
             )
 
         return np.array(
-            [spectrometer.detect(intensities[offset]) for offset in offsets]
+            [spectrometer.detect(intensities[offset]) for offset in offsets],
         )
 
 
