@@ -5,31 +5,27 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettingsNoRef
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cest_1hn_ip_ap"
 
+OFFSET_REF = 1e4
+
 
 class Cest1HnIpApSettings(CestSettings):
     name: Literal["cest_1hn_ip_ap"]
@@ -41,7 +37,6 @@ class Cest1HnIpApSettings(CestSettings):
     b1_inh_scale: float = 0.1
     b1_inh_res: int = 11
     eta_block: int = 0
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
@@ -56,7 +51,13 @@ class Cest1HnIpApSettings(CestSettings):
         return 0.5 * self.time_t1 / self.eta_block
 
 
-class Cest1HnIpApConfig(ExperimentConfig[Cest1HnIpApSettings, CestDataSettingsNoRef]):
+class Cest1HnIpApConfig(
+    ExperimentConfiguration[
+        Cest1HnIpApSettings,
+        ConditionsWithValidations,
+        CestDataSettingsNoRef,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -73,9 +74,9 @@ class Cest1HnIpApConfig(ExperimentConfig[Cest1HnIpApSettings, CestDataSettingsNo
 
 
 def build_spectrometer(
-    config: Cest1HnIpApConfig, spin_system: SpinSystem
+    config: Cest1HnIpApConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -98,11 +99,10 @@ class Cest1HnIpApSequence:
     settings: Cest1HnIpApSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
-
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
         spectrometer.offset_i = 0.0
 
@@ -115,7 +115,8 @@ class Cest1HnIpApSequence:
         start = d_taud @ spectrometer.get_start_magnetization(terms=["ie"])
         start = spectrometer.keep(start, components=["ie", "iz"])
 
-        intensities = {}
+        intensities: dict[float, float] = {}
+
         for offset in set(offsets):
             spectrometer.offset_i = offset
             if self.settings.eta_block > 0:
