@@ -4,31 +4,27 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettingsNoRef
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cest_ch3_1h_ip_ap"
 
+OFFSET_REF = 1e4
+
 
 class CestCh31HIpApSettings(CestSettings):
     name: Literal["cest_ch3_1h_ip_ap"]
@@ -39,7 +35,6 @@ class CestCh31HIpApSettings(CestSettings):
     b1_frq: float
     b1_inh_scale: float = 0.1
     b1_inh_res: int = 11
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
@@ -47,7 +42,11 @@ class CestCh31HIpApSettings(CestSettings):
 
 
 class CestCh31HIpApConfig(
-    ExperimentConfig[CestCh31HIpApSettings, CestDataSettingsNoRef]
+    ExperimentConfiguration[
+        CestCh31HIpApSettings,
+        ConditionsWithValidations,
+        CestDataSettingsNoRef,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
@@ -65,9 +64,9 @@ class CestCh31HIpApConfig(
 
 
 def build_spectrometer(
-    config: CestCh31HIpApConfig, spin_system: SpinSystem
+    config: CestCh31HIpApConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -90,11 +89,10 @@ class CestCh31HIpApSequence:
     settings: CestCh31HIpApSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
-
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
         spectrometer.offset_i = 0.0
 
@@ -106,7 +104,8 @@ class CestCh31HIpApSequence:
         start = d_d1 @ spectrometer.get_start_magnetization(terms=["ie"])
         start = spectrometer.keep(start, components=["ie", "iz"])
 
-        intensities = {}
+        intensities: dict[float, float] = {}
+
         for offset in set(offsets):
             spectrometer.offset_i = offset
             mag = spectrometer.pulse_i(self.settings.time_t1, 0.0) @ start
