@@ -5,32 +5,28 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettings
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.constants import get_multiplet
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "coscest_13c"
 
+OFFSET_REF = 1e4
+
 
 class CosCest13CSettings(CestSettings):
     name: Literal["coscest_13c"]
@@ -43,26 +39,30 @@ class CosCest13CSettings(CestSettings):
     b1_frq: float
     b1_inh_scale: float = 0.1
     b1_inh_res: int = 11
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
         return f"[iz_{self.observed_state}]"
 
 
-class CosCest13CConfig(ExperimentConfig[CosCest13CSettings, CestDataSettings]):
+class CosCest13CConfig(
+    ExperimentConfiguration[
+        CosCest13CSettings, ConditionsWithValidations, CestDataSettings
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
         return ToBeFitted(
-            rates=["r2_i", f"r1_i_{state}"], model_free=[f"tauc_{state}", f"s2_{state}"]
+            rates=["r2_i", f"r1_i_{state}"],
+            model_free=[f"tauc_{state}", f"s2_{state}"],
         )
 
 
 def build_spectrometer(
-    config: CosCest13CConfig, spin_system: SpinSystem
+    config: CosCest13CConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -89,10 +89,10 @@ class CosCest13CSequence:
     settings: CosCest13CSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
-    def _calc_cosine_shape(self, spectrometer: Spectrometer) -> np.ndarray:
+    def _calc_cosine_shape(self, spectrometer: Spectrometer) -> ArrayFloat:
         time_t1 = self.settings.time_t1
         sw = self.settings.sw
         cos_n = self.settings.cos_n
@@ -121,28 +121,28 @@ class CosCest13CSequence:
 
         if n_left:
             pulse_left = spectrometer.shaped_pulse_i(
-                n_left * dt, amplitudes[:n_left], phases[:n_left]
+                n_left * dt,
+                amplitudes[:n_left],
+                phases[:n_left],
             )
             pulse = pulse_left[phase_left] @ pulse
 
         return pulse
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
 
         start = spectrometer.get_equilibrium()
 
         d_eq = (
             spectrometer.delays(self.settings.time_equil)
-            if self.settings.time_equil > 0.0
+            if self.settings.time_equil > 0
             else spectrometer.identity
         )
 
-        intensities = {}
+        intensities: dict[float, ArrayFloat] = {}
 
         for offset in set(offsets):
-
             if self.is_reference(offset):
                 intensities[offset] = d_eq @ start
                 continue
@@ -152,7 +152,7 @@ class CosCest13CSequence:
             intensities[offset] = d_eq @ self._calc_cosine_shape(spectrometer) @ start
 
         return np.array(
-            [spectrometer.detect(intensities[offset]) for offset in offsets]
+            [spectrometer.detect(intensities[offset]) for offset in offsets],
         )
 
 
