@@ -5,31 +5,27 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettingsNoRef
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "coscest_1hn_ip_ap"
 
+OFFSET_REF = 1e4
+
 
 class CosCest1HnIpApSettings(CestSettings):
     name: Literal["coscest_1hn_ip_ap"]
@@ -43,7 +39,6 @@ class CosCest1HnIpApSettings(CestSettings):
     b1_frq: float
     b1_inh_scale: float = 0.1
     b1_inh_res: int = 11
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
@@ -51,7 +46,11 @@ class CosCest1HnIpApSettings(CestSettings):
 
 
 class CosCest1HnIpApConfig(
-    ExperimentConfig[CosCest1HnIpApSettings, CestDataSettingsNoRef]
+    ExperimentConfiguration[
+        CosCest1HnIpApSettings,
+        ConditionsWithValidations,
+        CestDataSettingsNoRef,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
@@ -69,9 +68,9 @@ class CosCest1HnIpApConfig(
 
 
 def build_spectrometer(
-    config: CosCest1HnIpApConfig, spin_system: SpinSystem
+    config: CosCest1HnIpApConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -94,10 +93,10 @@ class CosCest1HnIpApSequence:
     settings: CosCest1HnIpApSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
-    def _calc_cosine_shape(self, spectrometer: Spectrometer) -> np.ndarray:
+    def _calc_cosine_shape(self, spectrometer: Spectrometer) -> ArrayFloat:
         time_t1 = self.settings.time_t1
         sw = self.settings.sw
         cos_n = self.settings.cos_n
@@ -125,19 +124,20 @@ class CosCest1HnIpApSequence:
 
         if n_left:
             pulse_left = spectrometer.shaped_pulse_i(
-                n_left * dt, amplitudes[:n_left], phases[:n_left]
+                n_left * dt,
+                amplitudes[:n_left],
+                phases[:n_left],
             )
             pulse = pulse_left[phase_left] @ pulse
 
         return pulse
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
         spectrometer.offset_i = 0.0
 
         d_d1, d_taua, d_t1 = spectrometer.delays(
-            [self.settings.d1, self.settings.taua, self.settings.time_t1]
+            [self.settings.d1, self.settings.taua, self.settings.time_t1],
         )
 
         pp90_i = spectrometer.perfect90_i
@@ -146,7 +146,8 @@ class CosCest1HnIpApSequence:
         start = d_d1 @ spectrometer.get_start_magnetization(terms=["ie"])
         start = spectrometer.keep(start, components=["ie", "iz"])
 
-        intensities = {}
+        intensities: dict[float, float] = {}
+
         for offset in set(offsets):
             spectrometer.offset_i = offset
             if self.is_reference(offset):
@@ -155,6 +156,7 @@ class CosCest1HnIpApSequence:
             else:
                 cest = self._calc_cosine_shape(spectrometer)
             intensities[offset] = spectrometer.detect(cest @ start)
+
         return np.array([intensities[offset] for offset in offsets])
 
 
