@@ -5,28 +5,22 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cpmg_15n_ip"
 
@@ -37,7 +31,6 @@ class Cpmg15NIpSettings(CpmgSettings):
     carrier: float
     pw90: float
     time_equil: float = 0.0
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def t_neg(self) -> float:
@@ -48,7 +41,13 @@ class Cpmg15NIpSettings(CpmgSettings):
         return f"[iz_{self.observed_state}]"
 
 
-class Cpmg15NIpConfig(ExperimentConfig[Cpmg15NIpSettings, RelaxationDataSettings]):
+class Cpmg15NIpConfig(
+    ExperimentConfiguration[
+        Cpmg15NIpSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -56,9 +55,9 @@ class Cpmg15NIpConfig(ExperimentConfig[Cpmg15NIpSettings, RelaxationDataSettings
 
 
 def build_spectrometer(
-    config: Cpmg15NIpConfig, spin_system: SpinSystem
+    config: Cpmg15NIpConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -77,8 +76,8 @@ def build_spectrometer(
 class Cpmg15NIpSequence:
     settings: Cpmg15NIpSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
+        ncycs_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - self.settings.pw90
             for ncyc in ncycs_no_ref
@@ -89,13 +88,12 @@ class Cpmg15NIpSequence:
         delays = [self.settings.t_neg, self.settings.time_equil, *tau_cps.values()]
         return tau_cps, delays
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_neg = delays[self.settings.t_neg]
         d_eq = delays[self.settings.time_equil]
         d_cp = {ncyc: delays[delay] for ncyc, delay in tau_cps.items()}
@@ -114,22 +112,22 @@ class Cpmg15NIpSequence:
         intensities = {
             0.0: spectrometer.detect(d_eq @ p90[0] @ p180pmx @ p90[0] @ start),
             -1.0: spectrometer.detect(
-                part2 @ d_cp[-1.0] @ p180pmx @ d_cp[-1.0] @ part1
+                part2 @ d_cp[-1.0] @ p180pmx @ d_cp[-1.0] @ part1,
             ),
         }
         for ncyc in set(ncycs) - {0.0, -1.0}:
             echo = d_cp[ncyc] @ p180[1] @ d_cp[ncyc]
             cpmg = matrix_power(echo, int(ncyc))
             intensities[ncyc] = spectrometer.detect(
-                part2 @ cpmg @ p180pmx @ cpmg @ part1
+                part2 @ cpmg @ p180pmx @ cpmg @ part1,
             )
 
         # Return profile
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
