@@ -5,27 +5,23 @@ from functools import reduce
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
+from chemex.typing import ArrayBool, ArrayFloat, ArrayInt
 
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
 Delays = tuple[dict[float, float], dict[float, float], list[float]]
 
 
@@ -39,7 +35,6 @@ class Cpmg15N0013IpSettings(CpmgSettings):
     pw90: float
     time_equil: float = 0.0
     ncyc_max: int
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def t_neg(self) -> float:
@@ -55,7 +50,11 @@ class Cpmg15N0013IpSettings(CpmgSettings):
 
 
 class Cpmg15N0013IpConfig(
-    ExperimentConfig[Cpmg15N0013IpSettings, RelaxationDataSettings]
+    ExperimentConfiguration[
+        Cpmg15N0013IpSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
@@ -64,9 +63,9 @@ class Cpmg15N0013IpConfig(
 
 
 def build_spectrometer(
-    config: Cpmg15N0013IpConfig, spin_system: SpinSystem
+    config: Cpmg15N0013IpConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -85,8 +84,8 @@ def build_spectrometer(
 class Cpmg15N0013IpSequence:
     settings: Cpmg15N0013IpSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> Delays:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> Delays:
+        ncycs_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - 0.75 * self.settings.pw90
             for ncyc in ncycs_no_ref
@@ -108,23 +107,22 @@ class Cpmg15N0013IpSequence:
         ]
         return tau_cps, deltas, delays
 
-    def _get_phases(self, ncyc):
+    def _get_phases(self, ncyc: ArrayFloat) -> ArrayInt:
         cp_phases = np.array(
             [
                 [0, 0, 1, 3, 0, 0, 3, 1, 0, 0, 3, 1, 0, 0, 1, 3],
                 [1, 3, 2, 2, 3, 1, 2, 2, 3, 1, 2, 2, 1, 3, 2, 2],
-            ]
+            ],
         )
         indexes = np.flip(np.arange(2 * int(ncyc)))
         return np.take(cp_phases, indexes, mode="wrap", axis=1)
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, deltas, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_neg = delays[self.settings.t_neg]
         d_pos = delays[self.settings.t_pos]
         d_delta = {ncyc: delays[delay] for ncyc, delay in deltas.items()}
@@ -146,23 +144,23 @@ class Cpmg15N0013IpSequence:
                 @ d_pos
                 @ p180[[0, 1]]
                 @ p90[1]
-                @ start
-            )
+                @ start,
+            ),
         }
         for ncyc in set(ncycs) - {0.0}:
             phases = self._get_phases(ncyc)
             echo = d_cp[ncyc] @ p180 @ d_cp[ncyc]
             cpmg = reduce(np.matmul, echo[phases.T])
             intensities[ncyc] = spectrometer.detect(
-                d_delta[ncyc] @ p90[3] @ d_neg @ cpmg @ d_neg @ p90[1] @ start
+                d_delta[ncyc] @ p90[3] @ d_neg @ cpmg @ d_neg @ p90[1] @ start,
             )
 
         # Return profile
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
