@@ -5,28 +5,22 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cpmg_15n_tr"
 
@@ -39,7 +33,6 @@ class Cpmg15NTrSettings(CpmgSettings):
     time_equil: float = 0.0
     taub: float = 2.68e-3
     antitrosy: bool = False
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def t_neg(self) -> float:
@@ -53,11 +46,16 @@ class Cpmg15NTrSettings(CpmgSettings):
     def detection(self) -> str:
         if self.antitrosy:
             return f"[2izsz_{self.observed_state}] + [iz_{self.observed_state}]"
-        else:
-            return f"[2izsz_{self.observed_state}] - [iz_{self.observed_state}]"
+        return f"[2izsz_{self.observed_state}] - [iz_{self.observed_state}]"
 
 
-class Cpmg15NTrConfig(ExperimentConfig[Cpmg15NTrSettings, RelaxationDataSettings]):
+class Cpmg15NTrConfig(
+    ExperimentConfiguration[
+        Cpmg15NTrSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -72,9 +70,9 @@ class Cpmg15NTrConfig(ExperimentConfig[Cpmg15NTrSettings, RelaxationDataSettings
 
 
 def build_spectrometer(
-    config: Cpmg15NTrConfig, spin_system: SpinSystem
+    config: Cpmg15NTrConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -93,8 +91,8 @@ def build_spectrometer(
 class Cpmg15NTrSequence:
     settings: Cpmg15NTrSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
+        ncycs_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - self.settings.pw90
             for ncyc in ncycs_no_ref
@@ -107,13 +105,12 @@ class Cpmg15NTrSequence:
         ]
         return tau_cps, delays
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_neg = delays[self.settings.t_neg]
         d_eq = delays[self.settings.time_equil]
         d_taub = delays[self.settings.taub_eff]
@@ -149,8 +146,8 @@ class Cpmg15NTrSequence:
         return np.array([intst[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
