@@ -5,27 +5,23 @@ from functools import reduce
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
+from chemex.typing import ArrayBool, ArrayFloat, ArrayInt
 
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
 Delays = tuple[dict[float, float], dict[float, float], list[float]]
 
 
@@ -42,7 +38,6 @@ class Cpmg15NTr0013Settings(CpmgSettings):
     taub: float = 2.68e-3
     antitrosy: bool = False
     s3e: bool = True
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def t_neg(self) -> float:
@@ -56,12 +51,15 @@ class Cpmg15NTr0013Settings(CpmgSettings):
     def detection(self) -> str:
         if self.antitrosy:
             return f"[2izsz_{self.observed_state}] + [iz_{self.observed_state}]"
-        else:
-            return f"[2izsz_{self.observed_state}] - [iz_{self.observed_state}]"
+        return f"[2izsz_{self.observed_state}] - [iz_{self.observed_state}]"
 
 
 class Cpmg15NTr0013Config(
-    ExperimentConfig[Cpmg15NTr0013Settings, RelaxationDataSettings]
+    ExperimentConfiguration[
+        Cpmg15NTr0013Settings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
@@ -77,9 +75,9 @@ class Cpmg15NTr0013Config(
 
 
 def build_spectrometer(
-    config: Cpmg15NTr0013Config, spin_system: SpinSystem
+    config: Cpmg15NTr0013Config,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -98,8 +96,8 @@ def build_spectrometer(
 class Cpmg15NTr0013Sequence:
     settings: Cpmg15NTr0013Settings
 
-    def _get_delays(self, ncycs: np.ndarray) -> Delays:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> Delays:
+        ncycs_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - 0.75 * self.settings.pw90
             for ncyc in ncycs_no_ref
@@ -118,25 +116,25 @@ class Cpmg15NTr0013Sequence:
         ]
         return tau_cps, deltas, delays
 
-    def _get_phases(self, ncyc: float) -> tuple[np.ndarray, np.ndarray]:
+    def _get_phases(self, ncyc: float) -> tuple[ArrayInt, ArrayInt]:
         cp_phases1 = np.array(
             [
                 [1, 1, 0, 2, 1, 1, 2, 0, 1, 1, 2, 0, 1, 1, 0, 2],
                 [0, 2, 3, 3, 2, 0, 3, 3, 2, 0, 3, 3, 0, 2, 3, 3],
-            ]
+            ],
         )
         cp_phases2 = np.array(
             [
                 [0, 0, 1, 3, 0, 0, 3, 1, 0, 0, 3, 1, 0, 0, 1, 3],
                 [1, 3, 2, 2, 3, 1, 2, 2, 3, 1, 2, 2, 1, 3, 2, 2],
-            ]
+            ],
         )
         indexes = np.arange(int(ncyc))
         phases1 = np.take(cp_phases1, np.flip(indexes), mode="wrap", axis=1)
         phases2 = np.take(cp_phases2, indexes, mode="wrap", axis=1)
         return phases1, phases2
 
-    def _get_start(self, spectrometer: Spectrometer) -> np.ndarray:
+    def _get_start(self, spectrometer: Spectrometer) -> ArrayFloat:
         start = spectrometer.get_start_magnetization(["2izsz"])
         if self.settings.s3e:
             if self.settings.antitrosy:
@@ -146,13 +144,12 @@ class Cpmg15NTr0013Sequence:
             start *= 0.5
         return start
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, deltas, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_neg = delays[self.settings.t_neg]
         d_eq = delays[self.settings.time_equil]
         d_taub = delays[self.settings.taub_eff]
@@ -173,7 +170,8 @@ class Cpmg15NTr0013Sequence:
         else:
             p180_is = p90[0] @ p90[1] @ p180_sx @ p90[1] @ p90[0]
         palmer = np.mean(
-            p90[[0, 2]] @ d_taub @ p180_is @ d_taub @ p180_sx @ p90[[1, 3]], axis=0
+            p90[[0, 2]] @ d_taub @ p180_is @ d_taub @ p180_sx @ p90[[1, 3]],
+            axis=0,
         )
 
         # Calculating the instensities as a function of ncyc
@@ -187,8 +185,8 @@ class Cpmg15NTr0013Sequence:
                 @ p180[[0, 1]]
                 @ p90[0]
                 @ d_delta[0]
-                @ start
-            )
+                @ start,
+            ),
         }
         for ncyc in set(ncycs) - {0.0}:
             phases1, phases2 = self._get_phases(ncyc)
@@ -204,15 +202,15 @@ class Cpmg15NTr0013Sequence:
                 @ cpmg1
                 @ p90[0]
                 @ d_delta[ncyc]
-                @ start
+                @ start,
             )
 
         # Return profile
         return np.array([intst[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
