@@ -5,28 +5,22 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cpmg_1hn_ap"
 
@@ -36,8 +30,8 @@ class Cpmg1HnApSettings(CpmgSettings):
     time_t2: float
     carrier: float
     pw90: float
-    time_equil: float = 0.0
-    observed_state: Literal["a", "b", "c", "d"] = "a"
+    time_equil_1: float = 0.0
+    time_equil_2: float = 0.0
 
     @property
     def t_neg(self) -> float:
@@ -52,7 +46,13 @@ class Cpmg1HnApSettings(CpmgSettings):
         return f"[2izsz_{self.observed_state}]"
 
 
-class Cpmg1HnApConfig(ExperimentConfig[Cpmg1HnApSettings, RelaxationDataSettings]):
+class Cpmg1HnApConfig(
+    ExperimentConfiguration[
+        Cpmg1HnApSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -60,9 +60,9 @@ class Cpmg1HnApConfig(ExperimentConfig[Cpmg1HnApSettings, RelaxationDataSettings
 
 
 def build_spectrometer(
-    config: Cpmg1HnApConfig, spin_system: SpinSystem
+    config: Cpmg1HnApConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -81,8 +81,8 @@ def build_spectrometer(
 class Cpmg1HnApSequence:
     settings: Cpmg1HnApSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
+        ncycs_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - self.settings.pw90
             for ncyc in ncycs_no_ref
@@ -90,18 +90,23 @@ class Cpmg1HnApSequence:
         # An ncyc of -1 corresponds to the experiment without the CPMG
         # refocusisng pulses
         tau_cps[-1.0] = 0.5 * self.settings.time_t2
-        delays = [self.settings.t_neg, self.settings.time_equil, *tau_cps.values()]
+        delays = [
+            self.settings.t_neg,
+            self.settings.time_equil_1,
+            self.settings.time_equil_2,
+            *tau_cps.values(),
+        ]
         return tau_cps, delays
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_neg = delays[self.settings.t_neg]
-        d_eq = delays[self.settings.time_equil]
+        d_eq_1 = delays[self.settings.time_equil_1]
+        d_eq_2 = delays[self.settings.time_equil_2]
         d_cp = {ncyc: delays[delay] for ncyc, delay in tau_cps.items()}
 
         # Calculation of the spectrometers corresponding to all the pulses
@@ -113,25 +118,27 @@ class Cpmg1HnApSequence:
         start = spectrometer.get_start_magnetization(terms=self.settings.start)
 
         # Calculating the instensities as a function of ncyc
-        part1 = d_neg @ p90[0] @ start
-        part2 = d_eq @ p90[0] @ d_neg
+        part1 = d_neg @ p90[0] @ d_eq_1 @ start
+        part2 = d_eq_2 @ p90[0] @ d_neg
         intensities = {
-            0.0: spectrometer.detect(d_eq @ p90[0] @ p180pmx @ p90[0] @ start),
+            0.0: spectrometer.detect(
+                d_eq_2 @ p90[0] @ p180pmx @ p90[0] @ d_eq_1 @ start,
+            ),
             -1.0: spectrometer.detect(part2 @ d_cp[-1] @ p180pmx @ d_cp[-1] @ part1),
         }
         for ncyc in set(ncycs) - {0.0, -1.0}:
             echo = d_cp[ncyc] @ p180[1] @ d_cp[ncyc]
             cpmg = matrix_power(echo, int(ncyc))
             intensities[ncyc] = spectrometer.detect(
-                part2 @ cpmg @ p180pmx @ cpmg @ part1
+                part2 @ cpmg @ p180pmx @ cpmg @ part1,
             )
 
         # Return profile
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
