@@ -5,29 +5,22 @@ from functools import reduce
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayInt = NDArray[np.int_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat, ArrayInt
 
 EXPERIMENT_NAME = "cpmg_1hn_ap_0013"
 
@@ -44,7 +37,8 @@ class Cpmg1HnAp0013Settings(CpmgSettings):
     reburp_flg: bool = False
     pw_eburp: float = 1.4e-3
     pw_reburp: float = 1.52e-3
-    observed_state: Literal["a", "b", "c", "d"] = "a"
+    time_equil_1: float = 0.0
+    time_equil_2: float = 0.0
 
     @property
     def t_neg(self) -> float:
@@ -60,7 +54,11 @@ class Cpmg1HnAp0013Settings(CpmgSettings):
 
 
 class Cpmg1HnAp0013Config(
-    ExperimentConfig[Cpmg1HnAp0013Settings, RelaxationDataSettings]
+    ExperimentConfiguration[
+        Cpmg1HnAp0013Settings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
@@ -69,9 +67,9 @@ class Cpmg1HnAp0013Config(
 
 
 def build_spectrometer(
-    config: Cpmg1HnAp0013Config, spin_system: SpinSystem
+    config: Cpmg1HnAp0013Config,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -91,9 +89,10 @@ class Cpmg1HnAp0013Sequence:
     settings: Cpmg1HnAp0013Settings
 
     def _get_delays(
-        self, ncycs: NDArrayFloat
+        self,
+        ncycs: ArrayFloat,
     ) -> tuple[dict[float, float], dict[float, float], list[float]]:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+        ncycs_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - 0.75 * self.settings.pw90
             for ncyc in ncycs_no_ref
@@ -108,40 +107,43 @@ class Cpmg1HnAp0013Sequence:
             self.settings.t_neg,
             self.settings.pw_eburp,
             0.5 * self.settings.pw_reburp,
+            self.settings.time_equil_1,
+            self.settings.time_equil_2,
             *tau_cps.values(),
             *deltas.values(),
         ]
         return tau_cps, deltas, delays
 
-    def _get_phases(self, ncyc: NDArrayFloat) -> tuple[NDArrayInt, NDArrayInt]:
+    def _get_phases(self, ncyc: ArrayFloat) -> tuple[ArrayInt, ArrayInt]:
         cp_phases1 = np.array(
             [
                 [1, 1, 2, 0, 1, 1, 0, 2, 1, 1, 0, 2, 1, 1, 2, 0],
                 [2, 0, 3, 3, 0, 2, 3, 3, 0, 2, 3, 3, 2, 0, 3, 3],
-            ]
+            ],
         )
         cp_phases2 = np.array(
             [
                 [3, 3, 2, 0, 3, 3, 0, 2, 3, 3, 0, 2, 3, 3, 2, 0],
                 [2, 0, 1, 1, 0, 2, 1, 1, 0, 2, 1, 1, 2, 0, 1, 1],
-            ]
+            ],
         )
         indexes = np.arange(int(ncyc))
         phases1 = np.take(cp_phases1, np.flip(indexes), mode="wrap", axis=1)
         phases2 = np.take(cp_phases2, indexes, mode="wrap", axis=1)
         return phases1, phases2
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, deltas, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_neg = delays[self.settings.t_neg]
         d_taua = delays[self.settings.taua]
         d_eburp = delays[self.settings.pw_eburp]
         d_reburp = delays[0.5 * self.settings.pw_reburp]
+        d_eq_1 = delays[self.settings.time_equil_1]
+        d_eq_2 = delays[self.settings.time_equil_2]
         d_delta = {ncyc: delays[delay] for ncyc, delay in deltas.items()}
         d_cp = {ncyc: delays[delay] for ncyc, delay in tau_cps.items()}
 
@@ -151,7 +153,7 @@ class Cpmg1HnAp0013Sequence:
         pp90_i = spectrometer.perfect90_i
         pp180_isx = spectrometer.perfect180_i[0] @ spectrometer.perfect180_s[0]
 
-        # Calculation of the spectrometers for INEPT and purge elements
+        # Calculation of the propagators for INEPT and purge elements
         inept = pp90_i[1] @ d_taua @ pp180_isx @ d_taua @ pp90_i[0]
         zfilter = spectrometer.zfilter
 
@@ -163,16 +165,15 @@ class Cpmg1HnAp0013Sequence:
             p180pmy = p180[[1, 3]]
             pp90pmy = spectrometer.perfect90_i[[1, 3]]
             e180e_pmy = pp90pmy @ d_eburp @ p180pmy @ d_eburp @ pp90pmy
-            middle = [p180pmy @ e180e_pmy, e180e_pmy @ p180pmy]
+            middle = np.stack([p180pmy @ e180e_pmy, e180e_pmy @ p180pmy])
         elif self.settings.reburp_flg:
             pp180pmy = spectrometer.perfect180_i[[1, 3]]
             middle = d_reburp @ pp180pmy @ d_reburp
         else:
             middle = p180[[1, 3]]
-        middle = np.mean(middle, axis=0)
 
-        # Calculating the instensities as a function of ncyc
-        centre = {0.0: d_delta[0] @ p90[0] @ middle @ p90[0]}
+        # Calculating the intensities as a function of ncyc
+        centre = {0.0: d_eq_2 @ d_delta[0] @ p90[0] @ middle @ p90[0] @ d_eq_1}
 
         for ncyc in set(ncycs) - {0.0}:
             phases1, phases2 = self._get_phases(ncyc)
@@ -180,7 +181,16 @@ class Cpmg1HnAp0013Sequence:
             cpmg1 = reduce(np.matmul, echo[phases1.T])
             cpmg2 = reduce(np.matmul, echo[phases2.T])
             centre[ncyc] = (
-                d_delta[ncyc] @ p90[0] @ d_neg @ cpmg2 @ middle @ cpmg1 @ d_neg @ p90[0]
+                d_eq_2
+                @ d_delta[ncyc]
+                @ p90[0]
+                @ d_neg
+                @ cpmg2
+                @ middle
+                @ cpmg1
+                @ d_neg
+                @ p90[0]
+                @ d_eq_1
             )
 
         intst = {
@@ -199,8 +209,8 @@ class Cpmg1HnAp0013Sequence:
         return np.array([intst[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
