@@ -5,28 +5,24 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
-from chemex.configuration.experiment import CpmgSettingsEvenNcycs
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
+from chemex.configuration.experiment import (
+    CpmgSettingsEvenNcycs,
+)
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cpmg_ch3_13c_h2c"
 
@@ -38,7 +34,6 @@ class CpmgCh313CH2cSettings(CpmgSettingsEvenNcycs):
     pw90: float
     taub: float = 2.0e-3
     time_equil: float = 0.0
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def t_neg(self) -> float:
@@ -54,7 +49,11 @@ class CpmgCh313CH2cSettings(CpmgSettingsEvenNcycs):
 
 
 class CpmgCh313CH2cConfig(
-    ExperimentConfig[CpmgCh313CH2cSettings, RelaxationDataSettings]
+    ExperimentConfiguration[
+        CpmgCh313CH2cSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
@@ -63,9 +62,9 @@ class CpmgCh313CH2cConfig(
 
 
 def build_spectrometer(
-    config: CpmgCh313CH2cConfig, spin_system: SpinSystem
+    config: CpmgCh313CH2cConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -84,8 +83,8 @@ def build_spectrometer(
 class CpmgCh313CH2cSequence:
     settings: CpmgCh313CH2cSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
+        ncycs_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - self.settings.pw90
             for ncyc in ncycs_no_ref
@@ -98,13 +97,12 @@ class CpmgCh313CH2cSequence:
         ]
         return tau_cps, delays
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_neg = delays[self.settings.t_neg]
         d_eq = delays[self.settings.time_equil]
         d_taub = delays[self.settings.taub]
@@ -123,7 +121,7 @@ class CpmgCh313CH2cSequence:
 
         # Calculating the instensities as a function of ncyc
         intensities = {
-            0.0: spectrometer.detect(d_eq @ p90[1] @ palmer @ p90[0] @ start)
+            0.0: spectrometer.detect(d_eq @ p90[1] @ palmer @ p90[0] @ start),
         }
         part1 = d_neg @ p90[0] @ start
         part2 = d_eq @ p90[1] @ d_neg
@@ -137,8 +135,8 @@ class CpmgCh313CH2cSequence:
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
