@@ -5,28 +5,22 @@ from functools import reduce
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat, ArrayInt
 
 EXPERIMENT_NAME = "cpmg_ch3_1h_sq"
 
@@ -40,14 +34,19 @@ class CpmgCh31HSqSettings(CpmgSettings):
     taua: float = 2.0e-3
     comp180_flg: bool = True
     ipap_flg: bool = False
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
         return f"[iy_{self.observed_state}]"
 
 
-class CpmgCh31HSqConfig(ExperimentConfig[CpmgCh31HSqSettings, RelaxationDataSettings]):
+class CpmgCh31HSqConfig(
+    ExperimentConfiguration[
+        CpmgCh31HSqSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -62,9 +61,9 @@ class CpmgCh31HSqConfig(ExperimentConfig[CpmgCh31HSqSettings, RelaxationDataSett
 
 
 def build_spectrometer(
-    config: CpmgCh31HSqConfig, spin_system: SpinSystem
+    config: CpmgCh31HSqConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -83,9 +82,9 @@ def build_spectrometer(
 class CpmgCh31HSqSequence:
     settings: CpmgCh31HSqSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
         frac = 7.0 / 3.0 if self.settings.comp180_flg else 1.0
-        ncyc_no_ref = ncycs[ncycs > 0.0]
+        ncyc_no_ref = ncycs[ncycs > 0]
         tau_cps = {
             ncyc: (
                 self.settings.time_t2
@@ -97,7 +96,7 @@ class CpmgCh31HSqSequence:
         delays = [self.settings.taua, *tau_cps.values()]
         return tau_cps, delays
 
-    def _get_phases(self) -> tuple[np.ndarray, np.ndarray]:
+    def _get_phases(self) -> tuple[ArrayInt, ArrayInt]:
         cp_phases1 = np.array([[0, 1], [1, 0]])
         cp_phases2 = np.array([[0, 3], [3, 0]])
         indexes = np.arange(int(self.settings.ncyc_max))
@@ -105,13 +104,12 @@ class CpmgCh31HSqSequence:
         phases2 = np.take(cp_phases2, indexes, mode="wrap", axis=1)
         return phases1, phases2
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_cp = {ncyc: delays[delay] for ncyc, delay in tau_cps.items()}
         d_taua = delays[self.settings.taua]
 
@@ -136,12 +134,12 @@ class CpmgCh31HSqSequence:
         if self.settings.ipap_flg:
             intensities = {
                 0.0: spectrometer.detect(
-                    d_taua @ (p180pmy @ p180c_py + p180c_my @ p180pmy) @ d_taua @ start
-                )
+                    d_taua @ (p180pmy @ p180c_py + p180c_my @ p180pmy) @ d_taua @ start,
+                ),
             }
         else:
             intensities = {
-                0.0: spectrometer.detect(d_taua @ d_taua @ p180pmy @ p180c_py @ start)
+                0.0: spectrometer.detect(d_taua @ d_taua @ p180pmy @ p180c_py @ start),
             }
 
         phases1, phases2 = self._get_phases()
@@ -160,19 +158,19 @@ class CpmgCh31HSqSequence:
             centre = cpmg2 @ p180pmy @ cpmg1
             if self.settings.ipap_flg:
                 intensities[ncyc] = spectrometer.detect(
-                    d_taua @ (centre @ p180c_py + p180c_my @ centre) @ d_taua @ start
+                    d_taua @ (centre @ p180c_py + p180c_my @ centre) @ d_taua @ start,
                 )
             else:
                 intensities[ncyc] = spectrometer.detect(
-                    d_taua @ d_taua @ centre @ p180c_py @ start
+                    d_taua @ d_taua @ centre @ p180c_py @ start,
                 )
 
         # Return profile
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
