@@ -5,28 +5,22 @@ from functools import reduce
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat, ArrayInt
 
 EXPERIMENT_NAME = "cpmg_ch3_1h_tq"
 
@@ -39,14 +33,19 @@ class CpmgCh31HTqSettings(CpmgSettings):
     tauc: float = 0.67e-3  # ~ 1/(12*J[HC])
     comp180_flg: bool = True
     ipap_flg: bool = False
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
         return f"[2ixsz_{self.observed_state}]"
 
 
-class CpmgCh31HTqConfig(ExperimentConfig[CpmgCh31HTqSettings, RelaxationDataSettings]):
+class CpmgCh31HTqConfig(
+    ExperimentConfiguration[
+        CpmgCh31HTqSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -61,9 +60,9 @@ class CpmgCh31HTqConfig(ExperimentConfig[CpmgCh31HTqSettings, RelaxationDataSett
 
 
 def build_spectrometer(
-    config: CpmgCh31HTqConfig, spin_system: SpinSystem
+    config: CpmgCh31HTqConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -82,8 +81,8 @@ def build_spectrometer(
 class CpmgCh31HTqSequence:
     settings: CpmgCh31HTqSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
-        ncyc_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
+        ncyc_no_ref = ncycs[ncycs > 0]
         factor = 2.0 if self.settings.comp180_flg else 1.0
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - factor * self.settings.pw90
@@ -92,7 +91,7 @@ class CpmgCh31HTqSequence:
         delays = [self.settings.tauc, *tau_cps.values()]
         return tau_cps, delays
 
-    def _get_phases(self, ncyc: float) -> tuple[np.ndarray, np.ndarray]:
+    def _get_phases(self, ncyc: float) -> tuple[ArrayInt, ArrayInt]:
         cp_phases1 = [0, 1, 0, 1, 1, 0, 1, 0, 2, 3, 2, 3, 3, 2, 3, 2]
         cp_phases2 = [0, 3, 0, 3, 3, 0, 3, 0, 2, 1, 2, 1, 1, 2, 1, 2]
         indexes = np.arange(int(ncyc))
@@ -100,8 +99,7 @@ class CpmgCh31HTqSequence:
         phases2 = np.take(cp_phases2, indexes, mode="wrap")
         return phases1, phases2
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Getting the starting magnetization
@@ -109,7 +107,7 @@ class CpmgCh31HTqSequence:
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_cp = {ncyc: delays[delay] for ncyc, delay in tau_cps.items()}
         d_tauc = delays[self.settings.tauc]
 
@@ -150,8 +148,8 @@ class CpmgCh31HTqSequence:
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
