@@ -5,29 +5,23 @@ from functools import reduce
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.constants import GAMMA
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat, ArrayInt
 
 EXPERIMENT_NAME = "cpmg_ch3_1h_tq_diff"
 
@@ -43,7 +37,6 @@ class CpmgCh31HTqDiffSettings(CpmgSettings):
     tauc: float = 0.67e-3  # ~ 1/(12*J[HC])
     comp180_flg: bool = True
     ipap_flg: bool = False
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def k2_factor(self) -> float:
@@ -55,14 +48,19 @@ class CpmgCh31HTqDiffSettings(CpmgSettings):
 
 
 class CpmgCh31HTqDiffConfig(
-    ExperimentConfig[CpmgCh31HTqDiffSettings, RelaxationDataSettings]
+    ExperimentConfiguration[
+        CpmgCh31HTqDiffSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
 
         to_be_fitted = ToBeFitted(
-            rates=[f"r2_i_{state}", "d_"], model_free=[f"tauc_{state}", "d_"]
+            rates=[f"r2_i_{state}", "d_"],
+            model_free=[f"tauc_{state}", "d_"],
         )
 
         if self.experiment.ipap_flg:
@@ -73,9 +71,9 @@ class CpmgCh31HTqDiffConfig(
 
 
 def build_spectrometer(
-    config: CpmgCh31HTqDiffConfig, spin_system: SpinSystem
+    config: CpmgCh31HTqDiffConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -94,8 +92,8 @@ def build_spectrometer(
 class CpmgCh31HTqDiffSequence:
     settings: CpmgCh31HTqDiffSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
-        ncyc_no_ref = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
+        ncyc_no_ref = ncycs[ncycs > 0]
         factor = 2.0 if self.settings.comp180_flg else 1.0
         tau_cps = {
             ncyc: self.settings.time_t2 / (4.0 * ncyc) - factor * self.settings.pw90
@@ -104,7 +102,7 @@ class CpmgCh31HTqDiffSequence:
         delays = [self.settings.tau, *tau_cps.values()]
         return tau_cps, delays
 
-    def _get_phases(self, ncyc: float) -> tuple[np.ndarray, np.ndarray]:
+    def _get_phases(self, ncyc: float) -> tuple[ArrayInt, ArrayInt]:
         cp_phases1 = [0, 1, 0, 1, 1, 0, 1, 0, 2, 3, 2, 3, 3, 2, 3, 2]
         cp_phases2 = [0, 3, 0, 3, 3, 0, 3, 0, 2, 1, 2, 1, 1, 2, 1, 2]
         indexes = np.arange(int(ncyc))
@@ -112,8 +110,7 @@ class CpmgCh31HTqDiffSequence:
         phases2 = np.take(cp_phases2, indexes, mode="wrap")
         return phases1, phases2
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Getting the starting magnetization
@@ -141,7 +138,7 @@ class CpmgCh31HTqDiffSequence:
         # Calculation of the spectrometers with gradient 4
         spectrometer.gradient_dephasing = 4.0 * self.settings.k2_factor
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays_4 = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays_4 = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_cp_4 = {ncyc: delays_4[delay] for ncyc, delay in tau_cps.items()}
         d_tau_4 = delays_4[self.settings.tau]
         p180_4 = spectrometer.p180_i
@@ -185,8 +182,8 @@ class CpmgCh31HTqDiffSequence:
         return np.array([intst[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
