@@ -5,28 +5,22 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
 from chemex.configuration.experiment import CpmgSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "cpmg_ch3_mq"
 
@@ -36,14 +30,19 @@ class CpmgCh3MqSettings(CpmgSettings):
     time_t2: float
     t_zeta: float = 1.0 / (8.0 * 125.3)
     small_protein: bool = False
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
         return f"[2iysx_{self.observed_state}]"
 
 
-class CpmgCh3MqConfig(ExperimentConfig[CpmgCh3MqSettings, RelaxationDataSettings]):
+class CpmgCh3MqConfig(
+    ExperimentConfiguration[
+        CpmgCh3MqSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -51,9 +50,9 @@ class CpmgCh3MqConfig(ExperimentConfig[CpmgCh3MqSettings, RelaxationDataSettings
 
 
 def build_spectrometer(
-    config: CpmgCh3MqConfig, spin_system: SpinSystem
+    config: CpmgCh3MqConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -70,19 +69,18 @@ def build_spectrometer(
 class CpmgCh3MqSequence:
     settings: CpmgCh3MqSettings
 
-    def _get_delays(self, ncycs: np.ndarray) -> tuple[dict[float, float], list[float]]:
-        ncycs_no_refs = ncycs[ncycs > 0.0]
+    def _get_delays(self, ncycs: ArrayFloat) -> tuple[dict[float, float], list[float]]:
+        ncycs_no_refs = ncycs[ncycs > 0]
         tau_cps = {ncyc: self.settings.time_t2 / (4.0 * ncyc) for ncyc in ncycs_no_refs}
         delays = [self.settings.t_zeta, *tau_cps.values()]
         return tau_cps, delays
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps, all_delays = self._get_delays(ncycs)
-        delays = dict(zip(all_delays, spectrometer.delays(all_delays)))
+        delays = dict(zip(all_delays, spectrometer.delays(all_delays), strict=True))
         d_zeta = delays[self.settings.t_zeta]
         d_cp = {ncyc: delays[delay] for ncyc, delay in tau_cps.items()}
 
@@ -108,8 +106,8 @@ class CpmgCh3MqSequence:
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
