@@ -5,31 +5,30 @@ from functools import reduce
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
-from chemex.configuration.experiment import CpmgSettingsEvenNcycs
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
+from chemex.configuration.experiment import (
+    CpmgSettingsEvenNcycs,
+)
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CpmgPlotter
+from chemex.plotters.cpmg import CpmgPlotter
 from chemex.printers.data import CpmgPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat, ArrayInt
 
 EXPERIMENT_NAME = "cpmg_hn_dq_zq"
 
+NU_CPMG_LIMIT_1 = 51.0
+NU_CPMG_LIMIT_2 = 255.0
+
 
 class CpmgHNDqZqSettings(CpmgSettingsEvenNcycs):
     name: Literal["cpmg_hn_dq_zq"]
@@ -39,17 +38,21 @@ class CpmgHNDqZqSettings(CpmgSettingsEvenNcycs):
     pw90_h: float
     pw90_n: float
     dq_flg: bool
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
         if self.dq_flg:
             return f"[2ixsx_{self.observed_state}] - [2iysy_{self.observed_state}]"
-        else:
-            return f"[2ixsx_{self.observed_state}] + [2iysy_{self.observed_state}]"
+        return f"[2ixsx_{self.observed_state}] + [2iysy_{self.observed_state}]"
 
 
-class CpmgHNDqZqConfig(ExperimentConfig[CpmgHNDqZqSettings, RelaxationDataSettings]):
+class CpmgHNDqZqConfig(
+    ExperimentConfiguration[
+        CpmgHNDqZqSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
@@ -60,9 +63,9 @@ class CpmgHNDqZqConfig(ExperimentConfig[CpmgHNDqZqSettings, RelaxationDataSettin
 
 
 def build_spectrometer(
-    config: CpmgHNDqZqConfig, spin_system: SpinSystem
+    config: CpmgHNDqZqConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -83,22 +86,23 @@ def build_spectrometer(
 class CpmgHNDqZqSequence:
     settings: CpmgHNDqZqSettings
 
-    def _get_tau_cps(self, ncycs: np.ndarray) -> dict[float, float]:
-        ncycs_no_ref = ncycs[ncycs > 0.0]
+    def _get_tau_cps(self, ncycs: ArrayFloat) -> dict[float, float]:
+        ncycs_no_ref = ncycs[ncycs > 0]
         return dict(
             zip(
                 ncycs_no_ref,
                 self.settings.time_t2 / (4.0 * ncycs_no_ref)
                 - 7.0 / 3.0 * self.settings.pw90_n,
-            )
+                strict=True,
+            ),
         )
 
-    def _get_phases(self, ncyc: float) -> tuple[np.ndarray, np.ndarray]:
+    def _get_phases(self, ncyc: float) -> tuple[ArrayInt, ArrayInt]:
         nu_cpmg = ncyc / self.settings.time_t2
-        if nu_cpmg < 51.0:
+        if nu_cpmg < NU_CPMG_LIMIT_1:
             cp_phases1 = [0, 1, 0, 1]
             cp_phases2 = [1, 0, 1, 0]
-        elif nu_cpmg < 255.0:
+        elif nu_cpmg < NU_CPMG_LIMIT_2:
             cp_phases1 = [0]
             cp_phases2 = [1]
         else:
@@ -109,13 +113,13 @@ class CpmgHNDqZqSequence:
         phases2 = np.take(cp_phases2, np.flip(indexes), mode="wrap")
         return phases1, phases2
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         ncycs = data.metadata
 
         # Calculation of the spectrometers corresponding to all the delays
         tau_cps = self._get_tau_cps(ncycs)
-        d_cp = dict(zip(tau_cps.keys(), spectrometer.delays(list(tau_cps.values()))))
+        ncyc_list, delay_list = zip(*tau_cps.items(), strict=True)
+        d_cp = dict(zip(ncyc_list, spectrometer.delays(delay_list), strict=True))
 
         # Calculation of the spectrometers corresponding to all the pulses
         p9024090_1 = spectrometer.p9024090_nh_1[[0, 1], [0, 1]]
@@ -138,8 +142,8 @@ class CpmgHNDqZqSequence:
         return np.array([intensities[ncyc] for ncyc in ncycs])
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return metadata == 0.0
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return metadata == 0
 
 
 def register() -> None:
