@@ -5,32 +5,28 @@ from typing import Literal
 
 import numpy as np
 from numpy.linalg import matrix_power
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import CestDataSettings
 from chemex.configuration.experiment import CestSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ToBeFitted
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import CestFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.constants import get_multiplet
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import CestPlotter
+from chemex.plotters.cest import CestPlotter
 from chemex.printers.data import CestPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "dcest_13c"
 
+OFFSET_REF = 1e4
+
 
 class DCest13CSettings(CestSettings):
     name: Literal["dcest_13c"]
@@ -42,7 +38,6 @@ class DCest13CSettings(CestSettings):
     b1_frq: float
     b1_inh_scale: float = 0.1
     b1_inh_res: int = 11
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def pw_dante(self) -> float:
@@ -61,17 +56,21 @@ class DCest13CSettings(CestSettings):
         return f"[iz_{self.observed_state}]"
 
 
-class DCest13CConfig(ExperimentConfig[DCest13CSettings, CestDataSettings]):
+class DCest13CConfig(
+    ExperimentConfiguration[
+        DCest13CSettings, ConditionsWithValidations, CestDataSettings
+    ],
+):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
         return ToBeFitted(
-            rates=["r2_i", f"r1_i_{state}"], model_free=[f"tauc_{state}", f"s2_{state}"]
+            rates=["r2_i", f"r1_i_{state}"],
+            model_free=[f"tauc_{state}", f"s2_{state}"],
         )
 
 
 def build_spectrometer(config: DCest13CConfig, spin_system: SpinSystem) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -98,25 +97,23 @@ class DCest13CSequence:
     settings: DCest13CSettings
 
     @staticmethod
-    def is_reference(metadata: NDArrayFloat) -> NDArrayBool:
-        return np.abs(metadata) > 1e4
-
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
+    def is_reference(metadata: ArrayFloat) -> ArrayBool:
+        return np.abs(metadata) > OFFSET_REF
 
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         offsets = data.metadata
 
         start = spectrometer.get_equilibrium()
 
         d_eq = (
             spectrometer.delays(self.settings.time_equil)
-            if self.settings.time_equil > 0.0
+            if self.settings.time_equil > 0
             else spectrometer.identity
         )
 
-        intensities = {}
+        intensities: dict[float, ArrayFloat] = {}
 
         for offset in set(offsets):
-
             if self.is_reference(offset):
                 intensities[offset] = d_eq @ start
                 continue
@@ -131,7 +128,7 @@ class DCest13CSequence:
             )
 
         return np.array(
-            [spectrometer.detect(intensities[offset]) for offset in offsets]
+            [spectrometer.detect(intensities[offset]) for offset in offsets],
         )
 
 
