@@ -4,35 +4,28 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import RelaxationSettings
-from chemex.configuration.experiment import ToBeFitted
+from chemex.configuration.experiment import ExperimentSettings
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import RelaxationPlotter
+from chemex.plotters.relaxation import RelaxationPlotter
 from chemex.printers.data import RelaxationPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "relaxation_hznz"
 
 
-class RelaxationHzNzSettings(RelaxationSettings):
+class RelaxationHzNzSettings(ExperimentSettings):
     name: Literal["relaxation_hznz"]
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
@@ -40,7 +33,11 @@ class RelaxationHzNzSettings(RelaxationSettings):
 
 
 class RelaxationHzNzConfig(
-    ExperimentConfig[RelaxationHzNzSettings, RelaxationDataSettings]
+    ExperimentConfiguration[
+        RelaxationHzNzSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
@@ -52,9 +49,9 @@ class RelaxationHzNzConfig(
 
 
 def build_spectrometer(
-    config: RelaxationHzNzConfig, spin_system: SpinSystem
+    config: RelaxationHzNzConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -71,8 +68,7 @@ def build_spectrometer(
 class RelaxationHzNzSequence:
     settings: RelaxationHzNzSettings
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         times = data.metadata
 
         # Getting the starting magnetization
@@ -85,14 +81,14 @@ class RelaxationHzNzSequence:
         return np.array(
             [
                 spectrometer.detect(
-                    delay @ p180_s @ delay @ p180_i @ delay @ p180_s @ delay @ start
+                    delay @ p180_s @ delay @ p180_i @ delay @ p180_s @ delay @ start,
                 )
                 for delay in delays
-            ]
+            ],
         )
 
-    def is_reference(self, metadata: NDArrayFloat) -> NDArrayBool:
-        return np.full_like(metadata, False, dtype=np.bool_)
+    def is_reference(self, metadata: ArrayFloat) -> ArrayBool:
+        return np.full_like(metadata, fill_value=False, dtype=np.bool_)
 
 
 def register() -> None:
