@@ -4,35 +4,28 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration, ToBeFitted
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import RelaxationDataSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import RelaxationSettings
-from chemex.configuration.experiment import ToBeFitted
+from chemex.configuration.experiment import ExperimentSettings
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_relaxation_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import PlanesFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import RelaxationPlotter
+from chemex.plotters.relaxation import RelaxationPlotter
 from chemex.printers.data import RelaxationPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "relaxation_nz"
 
 
-class RelaxationNzSettings(RelaxationSettings):
+class RelaxationNzSettings(ExperimentSettings):
     name: Literal["relaxation_nz"]
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def detection(self) -> str:
@@ -40,20 +33,25 @@ class RelaxationNzSettings(RelaxationSettings):
 
 
 class RelaxationNzConfig(
-    ExperimentConfig[RelaxationNzSettings, RelaxationDataSettings]
+    ExperimentConfiguration[
+        RelaxationNzSettings,
+        ConditionsWithValidations,
+        RelaxationDataSettings,
+    ],
 ):
     @property
     def to_be_fitted(self) -> ToBeFitted:
         state = self.experiment.observed_state
         return ToBeFitted(
-            rates=[f"r1_i_{state}"], model_free=[f"tauc_{state}", f"s2_{state}"]
+            rates=[f"r1_i_{state}"],
+            model_free=[f"tauc_{state}", f"s2_{state}"],
         )
 
 
 def build_spectrometer(
-    config: RelaxationNzConfig, spin_system: SpinSystem
+    config: RelaxationNzConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     settings = config.experiment
     conditions = config.conditions
 
@@ -70,8 +68,7 @@ def build_spectrometer(
 class RelaxationNzSequence:
     settings: RelaxationNzSettings
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, data: Data) -> ArrayFloat:
         times = data.metadata
 
         # Getting the starting magnetization
@@ -81,8 +78,8 @@ class RelaxationNzSequence:
         delays = spectrometer.delays(times)
         return np.array([spectrometer.detect(delay @ start) for delay in delays])
 
-    def is_reference(self, metadata: NDArrayFloat) -> NDArrayBool:
-        return np.full_like(metadata, False, dtype=np.bool_)
+    def is_reference(self, metadata: ArrayFloat) -> ArrayBool:
+        return np.full_like(metadata, fill_value=False, dtype=np.bool_)
 
 
 def register() -> None:
