@@ -4,34 +4,28 @@ from dataclasses import dataclass
 from typing import Literal
 
 import numpy as np
-from numpy.typing import NDArray
 
+from chemex.configuration.base import ExperimentConfiguration
+from chemex.configuration.conditions import ConditionsWithValidations
 from chemex.configuration.data import ShiftDataSettings
-from chemex.configuration.experiment import ExperimentConfig
-from chemex.configuration.experiment import ShiftSettings
+from chemex.configuration.experiment import ExperimentSettings
 from chemex.containers.data import Data
 from chemex.containers.dataset import load_shift_dataset
-from chemex.experiments.factories import Creators
-from chemex.experiments.factories import factories
+from chemex.experiments.factories import Creators, factories
 from chemex.filterers import NoFilterer
 from chemex.nmr.basis import Basis
 from chemex.nmr.liouvillian import LiouvillianIS
 from chemex.nmr.spectrometer import Spectrometer
 from chemex.parameters.spin_system import SpinSystem
-from chemex.plotters import ShiftPlotter
+from chemex.plotters.shift import ShiftPlotter
 from chemex.printers.data import ShiftPrinter
-
-# Type definitions
-NDArrayFloat = NDArray[np.float_]
-NDArrayBool = NDArray[np.bool_]
-
+from chemex.typing import ArrayBool, ArrayFloat
 
 EXPERIMENT_NAME = "shift_15n_sqmq"
 
 
-class Shift15NSqMqSettings(ShiftSettings):
+class Shift15NSqMqSettings(ExperimentSettings):
     name: Literal["shift_15n_sqmq"]
-    observed_state: Literal["a", "b", "c", "d"] = "a"
 
     @property
     def cs_i_name(self) -> str:
@@ -42,14 +36,20 @@ class Shift15NSqMqSettings(ShiftSettings):
         return f"cs_s_{self.observed_state}"
 
 
-class Shift15NSqMqConfig(ExperimentConfig[Shift15NSqMqSettings, ShiftDataSettings]):
+class Shift15NSqMqConfig(
+    ExperimentConfiguration[
+        Shift15NSqMqSettings,
+        ConditionsWithValidations,
+        ShiftDataSettings,
+    ],
+):
     ...
 
 
 def build_spectrometer(
-    config: Shift15NSqMqConfig, spin_system: SpinSystem
+    config: Shift15NSqMqConfig,
+    spin_system: SpinSystem,
 ) -> Spectrometer:
-
     conditions = config.conditions
 
     basis = Basis(type="ixy_ixysxy", spin_system="nh")
@@ -57,7 +57,7 @@ def build_spectrometer(
     return Spectrometer(liouvillian)
 
 
-def _find_nearest(array, value):
+def _find_nearest(array: ArrayFloat, value: float) -> float:
     array = np.asarray(array)
     idx = (np.abs(array - value)).argmin()
     return array[idx]
@@ -67,8 +67,7 @@ def _find_nearest(array, value):
 class Shift15NSqMqSequence:
     settings: Shift15NSqMqSettings
 
-    def calculate(self, spectrometer: Spectrometer, data: Data) -> np.ndarray:
-
+    def calculate(self, spectrometer: Spectrometer, _data: Data) -> ArrayFloat:
         ref_shift_i = (
             spectrometer.par_values[self.settings.cs_i_name]
             * spectrometer.liouvillian.ppm_i
@@ -87,12 +86,12 @@ class Shift15NSqMqSequence:
             [
                 1e3
                 * (shift_sq - 0.5 * (shift_dq + shift_zq))
-                / spectrometer.liouvillian.ppm_i
-            ]
+                / spectrometer.liouvillian.ppm_i,
+            ],
         )
 
-    def is_reference(self, metadata: NDArrayFloat) -> NDArrayBool:
-        return np.full_like(metadata, False, dtype=np.bool_)
+    def is_reference(self, metadata: ArrayFloat) -> ArrayBool:
+        return np.full_like(metadata, fill_value=False, dtype=np.bool_)
 
 
 def register() -> None:
