@@ -1,39 +1,83 @@
 from __future__ import annotations
 
-from collections.abc import Iterable
-from collections.abc import Sequence
+from collections.abc import Iterable, Sequence
 from functools import reduce
+from itertools import product
 
 import numpy as np
 from cachetools import cached
 from cachetools.keys import hashkey
-from scipy.linalg import eig
-from scipy.linalg import eigvals
 from scipy.linalg import expm
-from scipy.linalg import inv
 
 from chemex.nmr.constants import Distribution
 from chemex.nmr.liouvillian import LiouvillianIS
+from chemex.typing import ArrayFloat
+
+DictArrayFloat = dict[str, ArrayFloat]
+
+# A small value used for numerical stability
+SMALL_VALUE = 1e-6
 
 
 def calculate_propagators(
-    liouv: np.ndarray, delays: float | Iterable[float], dephasing: bool = False
-) -> np.ndarray:
-    delays_ = np.asarray(delays).reshape(-1)
-    shape = liouv.shape
-    propagator_list = []
-    for a_liouvillian in liouv.reshape(-1, *shape[-2:]):
-        s, vr = eig(a_liouvillian)
-        vri = inv(vr)
-        if dephasing:
-            indexes = np.where(abs(s.imag) < 1e-6)[0]
-            vr, s, vri = vr[:, indexes], s[indexes], vri[indexes, :]
-        d = np.asarray([np.diag(np.exp(s * t)) for t in delays_])
-        propagator_list.append((vr @ d @ vri).real)
-    propagators = np.asarray(propagator_list).swapaxes(0, 1).reshape(-1, *shape)
+    liouv: ArrayFloat,
+    delays: float | Iterable[float],
+    dephasing: bool = False,
+) -> ArrayFloat:
+    """Calculate the propagators for the given delays.
+
+    Parameters
+    ----------
+    liouv : ArrayFloat
+        The Liouvillians of the system
+    delays : float | Iterable[float]
+        The delays to calculate the propagators for
+    dephasing : bool, optional
+        Whether to apply dephasing, by default False
+
+    Returns:
+    -------
+    ArrayFloat
+        The propagators
+    """
+    # Ensure delays is a 1D NumPy array
+    delays = np.asarray(delays).flatten()
+
+    if delays.size == 1 and not dephasing:
+        # If there's only one delay and no dephasing, calculate the propagator directly
+        # using scipy expm function
+        return expm(liouv * delays[0])
+
+    # Calculate eigenvalues and eigenvectors of the Liouvillian matrices
+    eigenvalues, eigenvectors = np.linalg.eig(liouv)
+
+    if dephasing:
+        # If dephasing is requested, adjust eigenvalues
+        eigenvalues = np.where(
+            abs(eigenvalues.imag) < SMALL_VALUE,
+            eigenvalues,
+            eigenvalues * 1e9,
+        )
+
+    # Calculate the exponentiated eigenvalues for each delay
+    evt = np.exp(np.multiply.outer(delays, eigenvalues))
+
+    # Create an empty array for diagonal matrices
+    diag = np.empty((*delays.shape, *liouv.shape), dtype=np.complex128)
+
+    # Calculate diagonal matrices for each delay
+    for index in product(*(range(x) for x in evt.shape[:-1])):
+        diag[index] = np.diag(evt[index])
+
+    # Calculate the propagators using matrix multiplication
+    propagators = eigenvectors @ diag @ np.linalg.inv(eigenvectors)
+
     if propagators.shape[0] == 1:
+        # If there's only one propagator, return it with no extra time dimension
         propagators = propagators[0]
-    return propagators
+
+    # Return the real part of the propagators
+    return propagators.real
 
 
 def _get_key(liouvillian: LiouvillianIS, *args, **kwargs):
@@ -41,7 +85,7 @@ def _get_key(liouvillian: LiouvillianIS, *args, **kwargs):
 
 
 @cached(cache={}, key=_get_key)
-def _make_perfect180(liouvillian: LiouvillianIS, spin: str) -> np.ndarray:
+def _make_perfect180(liouvillian: LiouvillianIS, spin: str) -> ArrayFloat:
     size = liouvillian.size
     identity = np.eye(size).reshape((1, 1, size, size))
     compx, compy, compz = (f"{spin}{axis}" for axis in "xyz")
@@ -57,7 +101,7 @@ def _make_perfect180(liouvillian: LiouvillianIS, spin: str) -> np.ndarray:
 
 
 @cached(cache={}, key=_get_key)
-def _make_perfect90(liouvillian: LiouvillianIS, spin: str) -> np.ndarray:
+def _make_perfect90(liouvillian: LiouvillianIS, spin: str) -> ArrayFloat:
     size = liouvillian.size
     zeros = np.zeros((size, size))
     rot = liouvillian.matrices.get(f"b1x_{spin}", zeros)
@@ -65,7 +109,7 @@ def _make_perfect90(liouvillian: LiouvillianIS, spin: str) -> np.ndarray:
 
 
 @cached(cache={}, key=_get_key)
-def _get_phases(liouvillian: LiouvillianIS) -> dict[str, np.ndarray]:
+def _get_phases(liouvillian: LiouvillianIS) -> DictArrayFloat:
     phases = {}
     size = liouvillian.size
     zeros = np.zeros((size, size))
@@ -76,7 +120,7 @@ def _get_phases(liouvillian: LiouvillianIS) -> dict[str, np.ndarray]:
 
 
 class Spectrometer:
-    def _add_phases(self, propagator: np.ndarray, spin: str = "i") -> np.ndarray:
+    def _add_phases(self, propagator: ArrayFloat, spin: str = "i") -> ArrayFloat:
         phases = self._phases[spin]
         return np.array([phases[i] @ propagator @ phases[-i] for i in range(4)])
 
@@ -102,7 +146,7 @@ class Spectrometer:
         self._p240_i = np.array(0.0)
         self._p180_s = np.array(0.0)
 
-    def keep(self, magnetization: np.ndarray, components: Iterable[str]) -> np.ndarray:
+    def keep(self, magnetization: ArrayFloat, components: Iterable[str]) -> ArrayFloat:
         return self.liouvillian.keep(magnetization, components)
 
     def update(self, par_values: dict[str, float]) -> None:
@@ -119,7 +163,7 @@ class Spectrometer:
         return self.liouvillian.carrier_i
 
     @carrier_i.setter
-    def carrier_i(self, value: float):
+    def carrier_i(self, value: float) -> None:
         self.liouvillian.carrier_i = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
@@ -129,7 +173,7 @@ class Spectrometer:
         return self.liouvillian.carrier_s
 
     @carrier_s.setter
-    def carrier_s(self, value: float):
+    def carrier_s(self, value: float) -> None:
         self.liouvillian.carrier_s = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
@@ -139,7 +183,7 @@ class Spectrometer:
         return self.liouvillian.offset_i
 
     @offset_i.setter
-    def offset_i(self, value: float):
+    def offset_i(self, value: float) -> None:
         self.liouvillian.offset_i = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
@@ -149,7 +193,7 @@ class Spectrometer:
         return self.liouvillian.offset_s
 
     @offset_s.setter
-    def offset_s(self, value: float):
+    def offset_s(self, value: float) -> None:
         self.liouvillian.offset_s = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
@@ -159,7 +203,7 @@ class Spectrometer:
         return self.liouvillian.b1_i
 
     @b1_i.setter
-    def b1_i(self, value: float):
+    def b1_i(self, value: float) -> None:
         self._pw90_i = 1.0 / (4.0 * value) if value else 0.0
         self.liouvillian.b1_i = value
         self.calculate_i_flag = True
@@ -170,7 +214,7 @@ class Spectrometer:
         return self.liouvillian.b1_s
 
     @b1_s.setter
-    def b1_s(self, value: float):
+    def b1_s(self, value: float) -> None:
         self._pw90_s = 1.0 / (4.0 * value) if value else 0.0
         self.liouvillian.b1_s = value
         self.calculate_i_flag = True
@@ -181,7 +225,7 @@ class Spectrometer:
         return self.liouvillian.b1_i_inh_scale
 
     @b1_i_inh_scale.setter
-    def b1_i_inh_scale(self, value: float):
+    def b1_i_inh_scale(self, value: float) -> None:
         self.liouvillian.b1_i_inh_scale = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
@@ -191,7 +235,7 @@ class Spectrometer:
         return self.liouvillian.b1_i_inh_res
 
     @b1_i_inh_res.setter
-    def b1_i_inh_res(self, value: int):
+    def b1_i_inh_res(self, value: int) -> None:
         self.liouvillian.b1_i_inh_res = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
@@ -201,17 +245,19 @@ class Spectrometer:
         return self.liouvillian.jeff_i
 
     @jeff_i.setter
-    def jeff_i(self, value: Distribution):
+    def jeff_i(self, value: Distribution) -> None:
         self.liouvillian.jeff_i = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
 
-    def get_equilibrium(self) -> np.ndarray:
+    def get_equilibrium(self) -> ArrayFloat:
         return self.liouvillian.get_equilibrium()
 
     def get_start_magnetization(
-        self, terms: Iterable[str], atom: str = "h"
-    ) -> np.ndarray:
+        self,
+        terms: Iterable[str],
+        atom: str = "h",
+    ) -> ArrayFloat:
         return self.liouvillian.get_start_magnetization(terms=terms, atom=atom)
 
     @property
@@ -219,10 +265,10 @@ class Spectrometer:
         return self.liouvillian.detection
 
     @detection.setter
-    def detection(self, value: str):
+    def detection(self, value: str) -> None:
         self.liouvillian.detection = value
 
-    def detect(self, magnetization: np.ndarray) -> float:
+    def detect(self, magnetization: ArrayFloat) -> float:
         return self.liouvillian.detect(magnetization)
 
     @property
@@ -230,12 +276,12 @@ class Spectrometer:
         return self.liouvillian.gradient_dephasing
 
     @gradient_dephasing.setter
-    def gradient_dephasing(self, value: float):
+    def gradient_dephasing(self, value: float) -> None:
         self.liouvillian.gradient_dephasing = value
         self.calculate_i_flag = True
         self.calculate_s_flag = True
 
-    def delays(self, times: float | Iterable[float]) -> np.ndarray:
+    def delays(self, times: float | Iterable[float]) -> ArrayFloat:
         return calculate_propagators(self.liouvillian.l_free, times)
 
     def pulse_i(
@@ -243,7 +289,7 @@ class Spectrometer:
         times: float | Iterable[float],
         phase: float,
         scale: float = 1.0,
-    ) -> np.ndarray:
+    ) -> ArrayFloat:
         dephased = self.b1_i_inh_scale == np.inf
         rad = phase * np.pi * 0.5
         liouv = (
@@ -258,7 +304,7 @@ class Spectrometer:
         times: float | Iterable[float],
         phase: float,
         scale: float = 1.0,
-    ) -> np.ndarray:
+    ) -> ArrayFloat:
         rad = phase * np.pi * 0.5
         liouv = (
             self.liouvillian.l_free
@@ -268,8 +314,11 @@ class Spectrometer:
         return calculate_propagators(liouv, times)
 
     def pulse_is(
-        self, times: float | Iterable[float], phase_i: float, phase_s: float
-    ) -> np.ndarray:
+        self,
+        times: float | Iterable[float],
+        phase_i: float,
+        phase_s: float,
+    ) -> ArrayFloat:
         dephased = self.b1_i_inh_scale == np.inf
         liouv = (
             self.liouvillian.l_free
@@ -281,10 +330,13 @@ class Spectrometer:
         return calculate_propagators(liouv, times, dephased)
 
     def shaped_pulse_i(
-        self, pw: float, amplitudes: Sequence[float], phases: Iterable[float]
-    ) -> np.ndarray:
+        self,
+        pw: float,
+        amplitudes: Sequence[float],
+        phases: Iterable[float],
+    ) -> ArrayFloat:
         time = pw / len(amplitudes)
-        pairs = list(zip(amplitudes, phases))
+        pairs = list(zip(amplitudes, phases, strict=True))
         pulses = {
             (amp, ph): self.pulse_i(time, ph, scale=amp) for amp, ph in set(pairs)
         }
@@ -299,34 +351,34 @@ class Spectrometer:
             self._p90_i, self._p180_i, self._p240_i = pulses.swapaxes(0, 1)
 
     @property
-    def p90_i(self) -> np.ndarray:
+    def p90_i(self) -> ArrayFloat:
         self._calculate_base_pulses_i()
         return self._p90_i
 
     @property
-    def p180_i(self) -> np.ndarray:
+    def p180_i(self) -> ArrayFloat:
         self._calculate_base_pulses_i()
         return self._p180_i
 
     @property
-    def p240_i(self) -> np.ndarray:
+    def p240_i(self) -> ArrayFloat:
         self._calculate_base_pulses_i()
         return self._p240_i
 
     @property
-    def p9018090_i_1(self) -> np.ndarray:
+    def p9018090_i_1(self) -> ArrayFloat:
         return self.p90_i[[3, 0, 1, 2]] @ self.p180_i @ self.p90_i[[3, 0, 1, 2]]
 
     @property
-    def p9018090_i_2(self) -> np.ndarray:
+    def p9018090_i_2(self) -> ArrayFloat:
         return self.p90_i[[1, 2, 3, 0]] @ self.p180_i @ self.p90_i[[1, 2, 3, 0]]
 
     @property
-    def p9024090_i_1(self) -> np.ndarray:
+    def p9024090_i_1(self) -> ArrayFloat:
         return self.p90_i[[3, 0, 1, 2]] @ self.p240_i @ self.p90_i[[3, 0, 1, 2]]
 
     @property
-    def p9024090_i_2(self) -> np.ndarray:
+    def p9024090_i_2(self) -> ArrayFloat:
         return self.p90_i[[1, 2, 3, 0]] @ self.p240_i @ self.p90_i[[1, 2, 3, 0]]
 
     def _calculate_base_pulses_s(self) -> None:
@@ -337,17 +389,18 @@ class Spectrometer:
             self._p180_s = pulses.swapaxes(0, 1)
 
     @property
-    def p180_s(self) -> np.ndarray:
+    def p180_s(self) -> ArrayFloat:
         self._calculate_base_pulses_s()
         return self._p180_s
 
-    def p9024090_nh(self, reverse: bool = False) -> np.ndarray:
+    def p9024090_nh(self, reverse: bool = False) -> ArrayFloat:
         ph_n = 1 if reverse else 3
         ph_h = 3 if reverse else 1
         pw240i, pw9024090i = np.array([8.0, 14.0]) * self._pw90_i / 3.0
         pw240s, pw9024090s = np.array([8.0, 14.0]) * self._pw90_s / 3.0
         t0, t1, t2, t3 = 0.5 * np.diff(
-            np.sort([pw240i, pw240s, pw9024090i, pw9024090s]), prepend=0.0
+            np.sort([pw240i, pw240s, pw9024090i, pw9024090s]),
+            prepend=0.0,
         )
         p0 = self.pulse_is(2.0 * t0, 0, 0)
         if pw9024090i <= pw9024090s:
@@ -370,21 +423,21 @@ class Spectrometer:
         return self._add_phases(self._add_phases(pw9024090is_xx, "s"), "i")
 
     @property
-    def p9024090_nh_1(self) -> np.ndarray:
+    def p9024090_nh_1(self) -> ArrayFloat:
         return self.p9024090_nh()
 
     @property
-    def p9024090_nh_2(self) -> np.ndarray:
+    def p9024090_nh_2(self) -> ArrayFloat:
         return self.p9024090_nh(reverse=True)
 
-    def calculate_shifts(self) -> np.ndarray:
+    def calculate_shifts(self) -> ArrayFloat:
         liouv = self.liouvillian.l_free.reshape(
-            (self.liouvillian.size, self.liouvillian.size)
+            (self.liouvillian.size, self.liouvillian.size),
         )
-        return eigvals(liouv).imag
+        return np.linalg.eigvals(liouv).imag
 
-    def offsets_to_ppms(self, offsets: np.ndarray) -> np.ndarray:
+    def offsets_to_ppms(self, offsets: ArrayFloat) -> ArrayFloat:
         return self.liouvillian.offsets_to_ppms(offsets)
 
-    def ppms_to_offsets(self, ppms: np.ndarray | float) -> np.ndarray | float:
+    def ppms_to_offsets(self, ppms: ArrayFloat | float) -> ArrayFloat | float:
         return self.liouvillian.ppms_to_offsets(ppms)
