@@ -8,21 +8,25 @@ from lmfit import Parameters as ParametersLF
 from scipy import stats
 
 from chemex.containers.experiments import Experiments
-from chemex.messages import print_chi2
-from chemex.messages import print_group_name
-from chemex.messages import print_making_plots
-from chemex.messages import print_writing_results
+from chemex.messages import (
+    print_chi2,
+    print_group_name,
+    print_making_plots,
+    print_plotting_canceled,
+    print_writing_results,
+)
 from chemex.parameters import database
 from chemex.printers.parameters import write_parameters
 
 
 def calculate_statistics(
-    experiments: Experiments, params_lf: ParametersLF
+    experiments: Experiments,
+    params_lf: ParametersLF,
 ) -> dict[str, int | float]:
     residuals = experiments.residuals(params_lf)
     ndata = len(residuals)
     nvarys = len(
-        [param for param in params_lf.values() if param.vary and not param.expr]
+        [param for param in params_lf.values() if param.vary and not param.expr],
     )
     chisqr = sum(residuals**2)
     redchi = chisqr / max(1, ndata - nvarys)
@@ -30,7 +34,7 @@ def calculate_statistics(
     aic = _neg2_log_likel + 2 * nvarys
     bic = _neg2_log_likel + np.log(ndata) * nvarys
     _, ks_p_value = stats.kstest(residuals, "norm")
-    pvalue: float = 1.0 - stats.chi2.cdf(chisqr, ndata - nvarys)  # type: ignore
+    pvalue: float = 1.0 - stats.chi2.cdf(chisqr, ndata - nvarys)
     return {
         "ndata": ndata,
         "nvarys": nvarys,
@@ -43,35 +47,25 @@ def calculate_statistics(
     }
 
 
-def _write_statistics(experiments: Experiments, path: Path):
+def _write_statistics(experiments: Experiments, path: Path) -> None:
     """Write fitting statistics to a file."""
     params_lf = database.build_lmfit_params(experiments.param_ids)
-    statistics = calculate_statistics(experiments, params_lf)
+    stats = calculate_statistics(experiments, params_lf)
     filename = path / "statistics.toml"
-    with open(filename, "w") as f:
-        f.write(f"\"number of data points\"                = {statistics['ndata']}\n")
-        f.write(f"\"number of variables\"                  = {statistics['nvarys']}\n")
+    with filename.open("w", encoding="utf-8") as f:
+        f.write(f"\"number of data points\"                = {stats['ndata']}\n")
+        f.write(f"\"number of variables\"                  = {stats['nvarys']}\n")
+        f.write(f"\"chi-square\"                           = {stats['chisqr']: .5e}\n")
+        f.write(f"\"reduced-chi-square\"                   = {stats['redchi']: .5e}\n")
+        f.write(f"\"chi-squared test\"                     = {stats['pvalue']: .5e}\n")
         f.write(
-            f"\"chi-square\"                           = {statistics['chisqr']: .5e}\n"
-        )
-        f.write(
-            f"\"reduced-chi-square\"                   = {statistics['redchi']: .5e}\n"
-        )
-        f.write(
-            f"\"chi-squared test\"                     = {statistics['pvalue']: .5e}\n"
-        )
-        f.write(
-            f"\"Kolmogorov-Smirnov test\"              = {statistics['ks_pvalue']: .5e}\n"
-        )
-        f.write(
-            f"\"Akaike Information Criterion (AIC)\"   = {statistics['aic']: .5e}\n"
-        )
-        f.write(
-            f"\"Bayesian Information Criterion (BIC)\" = {statistics['bic']: .5e}\n"
+            f"\"Kolmogorov-Smirnov test\"              = {stats['ks_pvalue']: .5e}\n",
         )
+        f.write(f"\"Akaike Information Criterion (AIC)\"   = {stats['aic']: .5e}\n")
+        f.write(f"\"Bayesian Information Criterion (BIC)\" = {stats['bic']: .5e}\n")
 
 
-def _write_files(experiments: Experiments, path: Path):
+def _write_files(experiments: Experiments, path: Path) -> None:
     """Write the results of the fit to output files."""
     print_writing_results(path)
     path.mkdir(parents=True, exist_ok=True)
@@ -80,7 +74,7 @@ def _write_files(experiments: Experiments, path: Path):
     _write_statistics(experiments, path=path)
 
 
-def _write_simulation_files(experiments: Experiments, path: Path):
+def _write_simulation_files(experiments: Experiments, path: Path) -> None:
     """Write the results of the simulation to output files."""
     print_writing_results(path)
     path.mkdir(parents=True, exist_ok=True)
@@ -88,9 +82,8 @@ def _write_simulation_files(experiments: Experiments, path: Path):
     experiments.write(path)
 
 
-def _write_plots(experiments: Experiments, path: Path):
+def _write_plots(experiments: Experiments, path: Path) -> None:
     """Plot the experimental and fitted data."""
-
     print_making_plots()
 
     path_ = path / "Plots"
@@ -98,13 +91,11 @@ def _write_plots(experiments: Experiments, path: Path):
     try:
         experiments.plot(path=path_)
     except KeyboardInterrupt:
-        print("  - Plotting cancelled\n")
-    print("")
+        print_plotting_canceled()
 
 
-def _write_simulation_plots(experiments: Experiments, path: Path):
+def _write_simulation_plots(experiments: Experiments, path: Path) -> None:
     """Plot the experimental and fitted data."""
-
     print_making_plots()
 
     path_ = path / "Plots"
@@ -112,15 +103,15 @@ def _write_simulation_plots(experiments: Experiments, path: Path):
     try:
         experiments.plot_simulation(path=path_)
     except KeyboardInterrupt:
-        print("  - Plotting cancelled\n")
-    print("")
+        print_plotting_canceled()
 
 
 def execute_post_fit(
     experiments: Experiments,
     path: Path,
+    *,
     plot: bool = False,
-):
+) -> None:
     _write_files(experiments, path)
     if plot:
         _write_plots(experiments, path)
@@ -129,8 +120,9 @@ def execute_post_fit(
 def execute_simulation(
     experiments: Experiments,
     path: Path,
+    *,
     plot: bool = False,
-):
+) -> None:
     experiments.prepare_for_simulation()
     _write_simulation_files(experiments, path)
     if plot:
@@ -142,7 +134,7 @@ def execute_post_fit_groups(experiments: Experiments, path: Path, plot: str) ->
     params_lf = database.build_lmfit_params(experiments.param_ids)
     statistics = calculate_statistics(experiments, params_lf)
     print_chi2(statistics["chisqr"], statistics["redchi"])
-    execute_post_fit(experiments, path / "All", plot != "nothing")
+    execute_post_fit(experiments, path / "All", plot=(plot != "nothing"))
 
 
 def print_header(grid: Iterable[str]) -> str:
@@ -157,9 +149,11 @@ def print_values(values: Iterable[float], chisqr: float) -> str:
 
 
 def print_values_stat(
-    params_lf: ParametersLF, fnames: Iterable[str], chisqr: float
+    params_lf: ParametersLF,
+    fnames: Iterable[str],
+    chisqr: float,
 ) -> str:
-    body_values_list = []
+    body_values_list: list[str] = []
     for fname in fnames:
         if fname in params_lf:
             body_values_list.append(f"{params_lf[fname].value:12.5e}")
