@@ -1,16 +1,11 @@
 from __future__ import annotations
 
 from contextlib import ExitStack
-from dataclasses import dataclass
-from dataclasses import field
+from dataclasses import dataclass, field
 from pathlib import Path
-from typing import Any
-from typing import Generic
-from typing import Protocol
-from typing import TypeVar
+from typing import Any, Generic, Protocol, TypeVar
 
 import numpy as np
-import numpy.typing as npt
 from matplotlib.axes import Axes
 from matplotlib.backends.backend_pdf import PdfPages
 from matplotlib.ticker import MaxNLocator
@@ -19,10 +14,9 @@ from chemex.containers.data import Data
 from chemex.containers.profile import Profile
 from chemex.messages import print_plot_filename
 from chemex.nmr.spectrometer import Spectrometer
-from chemex.plotters.plot import get_grid
-from chemex.plotters.plot import plot_profile
-from chemex.printers.plot import data_plot_printers
-from chemex.printers.plot import PlotPrinter
+from chemex.plotters.plot import get_grid, plot_profile
+from chemex.printers.plot import PlotPrinter, data_plot_printers
+from chemex.typing import ArrayFloat
 
 _GREY400 = "#BDBDBD"
 _LSTYLES = ("-", "--", "-.", ":")
@@ -39,7 +33,7 @@ class CestExperimentConfig(Protocol):
 T = TypeVar("T", bound=CestExperimentConfig)
 
 
-def estimate_sigma(values: npt.NDArray[np.float64]) -> float:
+def estimate_sigma(values: ArrayFloat) -> float:
     """Estimates standard deviation using median to exclude outliers.
 
     Up to 50% can be bad.
@@ -66,7 +60,7 @@ class CircularShift:
         sw_ppm, sw_ref = self.spectrometer.offsets_to_ppms(np.array([self.sw, 0.0]))
         self.sw_ppm = sw_ppm - sw_ref
 
-    def centre(self, array: np.ndarray, position: float) -> np.ndarray:
+    def centre(self, array: ArrayFloat, position: float) -> ArrayFloat:
         if self.sw == np.inf:
             return array
         cs_min = position - self.sw_ppm / 2.0
@@ -76,13 +70,18 @@ class CircularShift:
 def add_resonance_positions(
     ax1: Axes,
     ax2: Axes,
-    cs_values: np.ndarray,
+    cs_values: ArrayFloat,
     centre: float,
     circular_shift: CircularShift,
 ) -> tuple[Axes, Axes]:
     kwargs2 = {"color": _GREY400, "linewidth": 0.75, "zorder": -1}
     cs_shifted = circular_shift.centre(cs_values, centre)
-    for a_cs, a_cs_shifted, lstyle in zip(cs_values, cs_shifted, _LSTYLES):
+    for a_cs, a_cs_shifted, lstyle in zip(
+        cs_values,
+        cs_shifted,
+        _LSTYLES,
+        strict=False,
+    ):
         ax1.axvline(a_cs_shifted, linestyle=lstyle, **kwargs2)
         ax2.axvline(a_cs_shifted, linestyle=lstyle, **kwargs2)
         if a_cs != a_cs_shifted:
@@ -96,11 +95,14 @@ def plot_dcest(
     name: str,
     data_exp: Data,
     data_calc: Data,
-    cs_values: np.ndarray,
+    cs_values: ArrayFloat,
     circular_shift: CircularShift,
 ):
-    residuals = data_exp.exp - data_exp.calc
-    sigma = estimate_sigma(residuals)
+    if data_exp.size > 0:
+        residuals = data_exp.exp - data_exp.calc
+        sigma = estimate_sigma(residuals)
+    else:
+        sigma = 0.0
     centre = float(np.mean(cs_values))
     data_exp.metadata = circular_shift.centre(data_exp.metadata, centre)
     data_calc.metadata = circular_shift.centre(data_calc.metadata, centre)
@@ -121,6 +123,9 @@ def plot_dcest(
 
     ax1, ax2 = add_resonance_positions(ax1, ax2, cs_values, centre, circular_shift)
 
+    if data_exp.size == 0:
+        fig.delaxes(ax1)
+
     file_pdf.savefig(fig)
 
 
@@ -156,32 +161,32 @@ def create_plot_data_calc(profile: Profile) -> Data:
     filler = np.full_like(offsets_plot, 0.0)
 
     data_for_calculation = Data(exp=filler, err=filler, metadata=offsets_plot)
-    scale = profile.data.scale / np.mean(data.exp[refs])
-    data_for_calculation.calc = scale * profile.pulse_sequence.calculate(
-        spectrometer, data_for_calculation
+    data_for_calculation.calc = data.scale * profile.pulse_sequence.calculate(
+        spectrometer,
+        data_for_calculation,
     )
 
     ppms = spectrometer.offsets_to_ppms(offsets_plot)
+    intensity0 = np.mean(data.exp[refs])
     data_fit = Data(exp=filler, err=filler, metadata=ppms)
-    data_fit.calc = data_for_calculation.calc
+    data_fit.calc = data_for_calculation.calc / intensity0
 
     return data_fit
 
 
-def get_state_positions(spectrometer: Spectrometer) -> np.ndarray:
-
+def get_state_positions(spectrometer: Spectrometer) -> ArrayFloat:
     names = (f"cs_i_{state}" for state in "abcd")
     return np.array(
         [
             spectrometer.par_values[name]
             for name in names
             if name in spectrometer.par_values
-        ]
+        ],
     )
 
 
 class CestPlotter(Generic[T]):
-    def __init__(self, filename: Path, config: T, **_extra: Any):
+    def __init__(self, filename: Path, config: T, **_extra: Any) -> None:
         self.filename = filename
         self.config = config
         self.printer: PlotPrinter = data_plot_printers["cest"]
@@ -198,11 +203,15 @@ class CestPlotter(Generic[T]):
         dip_positions = get_state_positions(spectrometer)
         circular_shift = CircularShift(spectrometer, sw)
         plot_dcest(
-            pdf, str(profile.name), data_exp, data_calc, dip_positions, circular_shift
+            pdf,
+            str(profile.spin_system),
+            data_exp,
+            data_calc,
+            dip_positions,
+            circular_shift,
         )
 
     def plot(self, path: Path, profiles: list[Profile]) -> None:
-
         basename = path / self.filename.name
         name_pdf = basename.with_suffix(".pdf")
         name_exp = basename.with_suffix(".exp")
@@ -212,17 +221,20 @@ class CestPlotter(Generic[T]):
 
         with ExitStack() as stack:
             file_pdf = stack.enter_context(PdfPages(str(name_pdf)))
-            file_calc = stack.enter_context(name_fit.open("w"))
-            file_exp = stack.enter_context(name_exp.open("w"))
+            file_calc = stack.enter_context(name_fit.open("w", encoding="utf-8"))
+            file_exp = stack.enter_context(name_exp.open("w", encoding="utf-8"))
             for profile in sorted(profiles):
                 data_exp = create_plot_data_exp(profile)
                 data_calc = create_plot_data_calc(profile)
                 self._plot_profile(file_pdf, profile, data_exp, data_calc)
-                file_exp.write(self.printer.print_exp(str(profile.name), data_exp))
-                file_calc.write(self.printer.print_calc(str(profile.name), data_calc))
+                file_exp.write(
+                    self.printer.print_exp(str(profile.spin_system), data_exp),
+                )
+                file_calc.write(
+                    self.printer.print_calc(str(profile.spin_system), data_calc),
+                )
 
     def plot_simulation(self, path: Path, profiles: list[Profile]) -> None:
-
         basename = path / self.filename.name
         name_pdf = basename.with_suffix(".pdf")
         name_sim = basename.with_suffix(".sim")
@@ -231,9 +243,11 @@ class CestPlotter(Generic[T]):
 
         with ExitStack() as stack:
             file_pdf = stack.enter_context(PdfPages(str(name_pdf)))
-            file_sim = stack.enter_context(name_sim.open("w"))
+            file_sim = stack.enter_context(name_sim.open("w", encoding="utf-8"))
             for profile in sorted(profiles):
-                data_exp = create_plot_data_exp(profile)
+                data_exp = Data(np.array([]), np.array([]), np.array([]))
                 data_calc = create_plot_data_calc(profile)
                 self._plot_profile(file_pdf, profile, data_exp, data_calc)
-                file_sim.write(self.printer.print_calc(str(profile.name), data_calc))
+                file_sim.write(
+                    self.printer.print_calc(str(profile.spin_system), data_calc),
+                )
